#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary
from zope.traversing.interfaces import ITraversable

from pyams_app_msc.interfaces import MANAGE_THEATER_PERMISSION
from pyams_app_msc.shared.theater import ICinemaRoomContainer, ICinemaRoomContainerTarget, IMovieTheater
from pyams_app_msc.shared.theater.interfaces.room import CINEMA_ROOM_CONTAINER_KEY, ICinemaRoom, ROOMS_VOCABULARY
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config

from pyams_app_msc import _


@factory_config(ICinemaRoom)
class CinemaRoom(Persistent, Contained):
    """Cinema room persistent class"""

    active = FieldProperty(ICinemaRoom['active'])
    name = FieldProperty(ICinemaRoom['name'])
    capacity = FieldProperty(ICinemaRoom['capacity'])
    start_time = FieldProperty(ICinemaRoom['start_time'])
    end_time = FieldProperty(ICinemaRoom['end_time'])
    notepad = FieldProperty(ICinemaRoom['notepad'])


@factory_config(ICinemaRoomContainer)
class CinemaRoomContainer(BTreeOrderedContainer):
    """Cinema room container persistent class"""

    def get_active_items(self):
        """Active items iterator"""
        yield from filter(lambda x: x.active, self.values())


@adapter_config(required=ICinemaRoomContainerTarget,
                provides=ICinemaRoomContainer)
def cinema_room_container(context):
    """Cinema room container adapter"""
    return get_annotation_adapter(context, CINEMA_ROOM_CONTAINER_KEY,
                                  ICinemaRoomContainer,
                                  name='++room++')


@adapter_config(name='room',
                required=ICinemaRoomContainerTarget,
                provides=ITraversable)
class CinemaRoomContainerTraverser(ContextAdapter):
    """Cinema room container traverser"""

    def traverse(self, name, furtherPath):
        """Traverse target to rooms container"""
        return ICinemaRoomContainer(self.context)


@adapter_config(name='rooms',
                required=ICinemaRoomContainerTarget,
                provides=ISublocations)
class CinemaRoomSublocations(ContextAdapter):
    """Cinema rooms sub-locations"""

    def sublocations(self):
        """Sub-locations getter"""
        yield from ICinemaRoomContainer(self.context).values()


@adapter_config(required=ICinemaRoom,
                provides=IViewContextPermissionChecker)
class CinemaRoomPermissionChecker(ContextAdapter):
    """Cinema room permission checker"""

    edit_permission = MANAGE_THEATER_PERMISSION


@vocabulary_config(name=ROOMS_VOCABULARY)
class CinemaRoomsVocabulary(SimpleVocabulary):
    """Cinema rooms vocabulary"""

    def __init__(self, context):
        terms = []
        theater = get_parent(context, IMovieTheater)
        if theater is not None:
            request = check_request()
            translate = request.localizer.translate
            terms = sorted([
                SimpleTerm(item.__name__,
                           title=translate(_("{} ({} seats)")).format(item.name, item.capacity))
                for item in ICinemaRoomContainer(theater).get_active_items()
            ], key=lambda x: x.title)
        super().__init__(terms)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from cornice import Service
from cornice.validators import colander_validator
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPOk

from pyams_app_msc.shared.theater import IMovieTheater
from pyams_app_msc.shared.theater.api.interfaces import MSC_PRICE_API_ROUTE
from pyams_app_msc.shared.theater.api.schema import PriceGetterRequest, PriceGetterResponse
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPriceContainer
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_utils.registry import query_utility
from pyams_utils.rest import STATUS, http_error, rest_responses


price_service = Service(name=MSC_PRICE_API_ROUTE,
                        pyramid_route=MSC_PRICE_API_ROUTE,
                        description="Price getter service")


@price_service.options(validators=(check_cors_origin, set_cors_headers))
def price_options(request):
    """Price service options"""
    return ''


price_get_responses = rest_responses.copy()
price_get_responses[HTTPOk.code] = PriceGetterResponse(
    description="Price getter result")


@price_service.get(permission=USE_INTERNAL_API_PERMISSION,
                   schema=PriceGetterRequest(),
                   validators=(check_cors_origin, colander_validator, set_cors_headers),
                   response_schemas=price_get_responses)
def get_price(request):
    """Get price information"""
    theater_name = request.matchdict.get('theater_name')
    if not theater_name:
        return http_error(request, HTTPBadRequest)
    theater = query_utility(IMovieTheater, name=theater_name)
    if theater is None:
        return http_error(request, HTTPNotFound, "Unknown theater!")
    price_id = request.params.get('price_id')
    if not price_id:
        return http_error(request, HTTPBadRequest)
    price = ICinemaPriceContainer(theater).get(price_id)
    if price is None:
        return http_error(request, HTTPNotFound, "Price not found!")
    return {
        'status': STATUS.SUCCESS.value,
        'price': {
            'id': price.__name__,
            'active': price.active,
            'name': price.name,
            'participant_price': price.participant_price,
            'accompanying_price': price.accompanying_price,
            'accompanying_ratio': price.accompanying_ratio,
            'comment': price.comment
        }
    }



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from colander import Boolean, Float, Int, MappingSchema, SchemaNode, String, drop

from pyams_utils.rest import BaseResponseSchema


class PriceGetterQuery(MappingSchema):
    """Price getter query schema"""
    price_id = SchemaNode(String(),
                          description="Price ID")


class PriceGetterRequest(MappingSchema):
    """Price getter request"""
    querystring = PriceGetterQuery()


class PriceInfo(MappingSchema):
    """Price info schema"""
    id = SchemaNode(String(),
                    description="Price ID")
    active = SchemaNode(Boolean(),
                        description="Active price flag")
    name = SchemaNode(String(),
                      description="Price name")
    participant_price = SchemaNode(Float(),
                                   description="Participant price")
    accompanying_price = SchemaNode(Float(),
                                    description="Accompanying price")
    accompanying_ratio = SchemaNode(Int(),
                                    description="Accompanying ratio")
    comment = SchemaNode(String(),
                         description="Comments",
                         missing=drop)


class PriceGetterResult(BaseResponseSchema):
    """Price getter result schema"""
    price = PriceInfo()


class PriceGetterResponse(MappingSchema):
    """Price getter response"""
    body = PriceGetterResult()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.feature.booking.interfaces import IBookingArchiverTask
from pyams_app_msc.feature.booking.task import BookingArchiverTask
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_scheduler.interfaces import ITaskContainer, MANAGE_TASKS_PERMISSION
from pyams_scheduler.task.zmi import BaseTaskAddForm, BaseTaskEditForm
from pyams_scheduler.zmi import TaskContainerTable
from pyams_skin.viewlet.menu import MenuItem
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager

from pyams_app_msc import _


@viewlet_config(name='add-booking-archiver-task.menu',
                context=ITaskContainer, layer=IAdminLayer, view=TaskContainerTable,
                manager=IContextAddingsViewletManager, weight=220,
                permission=MANAGE_TASKS_PERMISSION)
class BookingArchiverTaskAddMenu(MenuItem):
    """Booking archiver task add menu"""

    label = _("Add booking archiver task...")
    href = 'add-booking-archiver-task.html'
    modal_target = True


@ajax_form_config(name='add-booking-archiver-task.html',
                  context=ITaskContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TASKS_PERMISSION)
class BookingArchiverTaskAddForm(BaseTaskAddForm):
    """Booking archiver task add form"""

    content_factory = IBookingArchiverTask
    content_label = BookingArchiverTask.label


@ajax_form_config(name='properties.html',
                  context=IBookingArchiverTask, layer=IPyAMSLayer,
                  permission=MANAGE_TASKS_PERMISSION)
class BookingArchiverTaskEditForm(BaseTaskEditForm):
    """Booking archiver task edit form"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from elasticsearch_dsl import Q
from hypatia.interfaces import ICatalog
from hypatia.query import Any

from pyams_content.shared.view.interfaces import IWfView
from pyams_content.shared.view.interfaces.query import IViewUserQuery
from pyams_content_es.shared.view.interfaces import IEsViewUserQuery
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_utility


#
# Audience query adapters
#

@adapter_config(name='audience',
                required=IWfView,
                provides=IViewUserQuery)
class CatalogAudienceQuery(ContextAdapter):
    """Catalog audience query"""

    @staticmethod
    def get_user_params(request):
        """User params getter"""
        audiences = request.params.getall('audience')
        if not audiences:
            return
        catalog = get_utility(ICatalog)
        yield Any(catalog['catalog_audience'], [
            value
            for audience in audiences
            for value in audience.split(',')
        ])


@adapter_config(name='audience',
                required=IWfView,
                provides=IEsViewUserQuery)
class EsIndexAudienceQuery(ContextAdapter):
    """Elasticsearch index audience query"""

    @staticmethod
    def get_user_params(request):
        """User params getter"""
        audiences = request.params.getall('audience')
        if not audiences:
            return
        yield Q('terms',
                **{'catalog_info.audiences': [
                    value
                    for audience in audiences
                    for value in audience.split(',')
                ]})


#
# Duration query adapters
#

@adapter_config(name='duration',
                required=IWfView,
                provides=IEsViewUserQuery)
class EsCatalogDurationQuery(ContextAdapter):
    """Elasticsearch index duration query"""

    @staticmethod
    def get_user_params(request):
        """User params getter"""
        duration = request.params.get('duration')
        if not duration:
            return
        min, max = duration.split('-')
        settings = {}
        if min:
            settings['gte'] = min
        if max:
            settings['lt'] = max
        yield Q('range',
                **{'catalog_info.duration': settings})



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.csrf import get_csrf_token
from zope.interface import Interface

from pyams_app_msc.skin import IPyAMSMSCLayer
from pyams_form.interfaces.form import IForm
from pyams_skin.interfaces.metas import IHTMLContentMetas
from pyams_skin.metas import ContentMeta
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config

__docformat__ = 'restructuredtext'


@adapter_config(name='csrf',
                required=(Interface, IPyAMSMSCLayer, IForm),
                provides=IHTMLContentMetas)
class CSRFContentMetas(ContextRequestViewAdapter):
    """CSRF content metas"""

    def get_metas(self):
        yield ContentMeta('csrf-param', value='csrf_token')
        yield ContentMeta('csrf-token', value=get_csrf_token(self.request))



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from pyams_app_msc.shared.catalog import ICatalogEntryInfo, IWfCatalogEntry
from pyams_content_api.shared.common.interfaces import IContentAPIInfo
from pyams_utils.adapter import adapter_config


@adapter_config(name='catalog_entry',
                required=IWfCatalogEntry,
                provides=IContentAPIInfo)
def catalog_entry_api_info(context):
    """Catalog entry API info"""
    info = ICatalogEntryInfo(context)
    return {
        'duration': info.duration
    }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Or
from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPForbidden
from zope.interface import Interface
from zope.principalannotation.interfaces import IPrincipalAnnotationUtility

from pyams_app_msc.feature.profile import IUserProfile, USER_PROFILE_KEY
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import LOGIN_REFERER_KEY
from pyams_security.interfaces.plugin import IAuthenticatedPrincipalEvent
from pyams_security.utility import get_principal
from pyams_security_views.interfaces.login import ILoginView
from pyams_site.interfaces import ISiteRoot
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.list import next_from
from pyams_utils.registry import get_utility, query_utility
from pyams_utils.url import absolute_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@adapter_config(required=(Interface, IPyAMSLayer, ILoginView),
                provides=IAJAXFormRenderer)
class LoginFormAJAXRenderer(ContextRequestViewAdapter):
    """Login form renderer"""

    def render(self, changes):  # pylint: disable=unused-argument
        """AJAX form renderer"""
        status = {
            'status': 'redirect'
        }
        request = self.request
        session = request.session
        hash = request.params.get('login_form.widgets.hash', '')
        if hash:
            if LOGIN_REFERER_KEY in session:
                status['location'] = f"{session[LOGIN_REFERER_KEY] or '/'}{hash}"
                del session[LOGIN_REFERER_KEY]
            else:
                status['location'] = f'/{hash}'
        else:
            principal_id = self.view.finished_state.get('changes')
            if principal_id:
                catalog = get_utility(ICatalog)
                query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                            Or(Eq(catalog['role:msc:manager'], principal_id),
                               Eq(catalog['role:msc:operator'], principal_id),
                               Eq(catalog['role:msc:contributor'], principal_id),
                               Eq(catalog['role:msc:reader'], principal_id)))
                theaters = list(CatalogResultSet(CatalogQuery(catalog).query(query)))
                if len(theaters) == 1:
                    theater = theaters[0]
                    status['location'] = absolute_url(theater, request, 'admin')
                elif len(theaters) > 1:
                    status['location'] = absolute_url(request.root, request,
                                                      'select-admin-theater.html')
                else:
                    theater = None
                    # request.principal is reified, so we can't use request.principal just
                    # after authentication to load profile!
                    profile_info = get_utility(IPrincipalAnnotationUtility) \
                        .getAnnotationsById(principal_id) \
                        .get(USER_PROFILE_KEY)
                    if (profile_info is not None) and profile_info.local_theaters:
                        theater_name = next_from(profile_info.local_theaters)
                        theater = query_utility(IMovieTheater, name=theater_name)
                    session = request.session
                    if theater is not None:
                        status['location'] = absolute_url(theater, request)
                        if LOGIN_REFERER_KEY in session:
                            del session[LOGIN_REFERER_KEY]
                    else:
                        if LOGIN_REFERER_KEY in session:
                            status['location'] = session[LOGIN_REFERER_KEY]
                            del session[LOGIN_REFERER_KEY]
                        else:
                            status['location'] = absolute_url(request.context, request)
        return status


@subscriber(IAuthenticatedPrincipalEvent)
def handle_authenticated_principal(event):
    """Check for inactive profile when authenticated"""
    principal = get_principal(principal_id=event.principal_id)
    profile = IUserProfile(principal)
    if not profile.active:
        raise HTTPForbidden(_("User profile is disabled!"))


@pagelet_config(name='select-admin-theater.html',
                context=ISiteRoot, layer=IPyAMSLayer)
@template_config(template='templates/select-admin-theater.pt',
                 layer=IPyAMSLayer)
class TheaterSelectionView(PortalContextIndexPage):
    """Theater selection view"""

    def get_theaters(self):
        """Theaters list getter"""
        principal_id = self.request.principal.id
        catalog = get_utility(ICatalog)
        query = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                    Or(Eq(catalog['role:msc:manager'], principal_id),
                       Eq(catalog['role:msc:operator'], principal_id),
                       Eq(catalog['role:msc:contributor'], principal_id),
                       Eq(catalog['role:msc:reader'], principal_id)))
        yield from sorted(CatalogResultSet(CatalogQuery(catalog).query(query)),
                          key=lambda x: II18n(x).query_attribute('title', request=self.request))



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import math
import os.path
from datetime import datetime, timezone
from io import BytesIO

from PIL import Image
from reportlab.lib.colors import black
from reportlab.lib.enums import TA_CENTER, TA_RIGHT
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import ParagraphStyle, getSampleStyleSheet
from reportlab.lib.units import cm
from reportlab.lib.utils import ImageReader
from reportlab.pdfbase.pdfmetrics import registerFont
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import BaseDocTemplate, Flowable, Frame, PageTemplate, Paragraph, Table, TableStyle

from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.feature.profile import IUserProfile
from pyams_app_msc.shared.catalog import IWfCatalogEntry
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPriceContainer
from pyams_i18n.interfaces import II18n
from pyams_security.utility import get_principal
from pyams_utils.date import SH_DATE_FORMAT, format_date, format_datetime
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.request import query_request
from pyams_utils.text import get_text_parts

__docformat__ = 'restructuredtext'

from pyams_app_msc import _
from pyams_utils.timezone import tztime

dirname = os.path.dirname(__file__)
registerFont(TTFont('Roboto-Light', f"{dirname}/fonts/Roboto-Light.ttf"))
registerFont(TTFont('Roboto-Medium', f"{dirname}/fonts/Roboto-Medium.ttf"))
registerFont(TTFont('Roboto-Bold', f"{dirname}/fonts/Roboto-Bold.ttf"))


STYLESHEET = getSampleStyleSheet()
STYLESHEET.add(ParagraphStyle(name='header',
                              fontName='Roboto-Light',
                              fontSize=11,
                              leading=15,
                              alignment=TA_RIGHT,
                              alias='header'))
STYLESHEET.add(ParagraphStyle(name='banner',
                              fontName='Roboto-Bold',
                              fontSize=15,
                              leading=24,
                              alignment=TA_CENTER,
                              alias='banner'))
STYLESHEET.add(ParagraphStyle(name='frame',
                              fontName='Roboto-Medium',
                              fontSize=12,
                              leading=16,
                              alignment=TA_CENTER,
                              alias='frame'))
STYLESHEET.add(ParagraphStyle(name='p',
                              fontName='Roboto-Light',
                              fontSize=10,
                              leading=15,
                              alias='p'))
STYLESHEET.add(ParagraphStyle(name='small',
                              fontName='Roboto-Light',
                              fontSize=8,
                              leading=9,
                              alias='small'))
STYLESHEET.add(ParagraphStyle(name='tiny',
                              fontName='Roboto-Light',
                              fontSize=7,
                              leading=8,
                              alias='tiny'))


class Quotation:
    """Quotation document"""

    class Template(PageTemplate):
        """Base quotation template"""

        def __init__(self, parent):
            self.parent = parent
            content = Frame(2.*cm,
                            2.*cm,
                            parent.document.pagesize[0] - 4.*cm,
                            parent.document.pagesize[1] - 4.*cm)
            super().__init__("Document", [content])

        def beforeDrawPage(self, canvas, document):
            canvas.saveState()
            try:
                pass
            finally:
                canvas.restoreState()

    class Header(Flowable):
        """Custom flowable header"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                theater = self.parent.theater
                settings = IMovieTheaterSettings(theater)
                logo = settings.quotation_logo
                if logo:
                    image = Image.open(BytesIO(logo.data))
                    self.canv.drawImage(ImageReader(image), 0, -4*cm, width=8*cm, height=5*cm,
                                        preserveAspectRatio=True)
                header = [II18n(theater).query_attribute('title', request=self.request)]
                address = theater.address
                if address:
                    if address.street:
                        header.append(address.street)
                    if address.locality:
                        header.append(address.locality)
                    header.append(f"{address.postal_code or ''} {address.city or ''}")
                header.append('')
                if theater.phone_number:
                    header.append(theater.phone_number)
                if theater.web_address:
                    header.append(theater.web_address)
                if theater.contact_email:
                    header.append(theater.contact_email)
                paragraph = Paragraph(f"{'<br />'.join(header)}", STYLESHEET['header'])
                paragraph.wrap(9*cm, 5*cm)
                paragraph.drawOn(self.canv, 8*cm, -4*cm)
            finally:
                self.canv.restoreState()

    class SubHeader(Flowable):
        """Custom sub-header"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                settings = IMovieTheaterSettings(self.parent.theater)
                self.canv.setFillColor(settings.get_quotation_color())
                self.canv.setStrokeColor(settings.get_quotation_color())
                self.canv.rect(0, -6*cm, 17*cm, 1*cm, stroke=1, fill=1)
                paragraph = Paragraph(self.parent.translate(_("QUOTATION")), STYLESHEET['banner'])
                paragraph.wrap(17*cm, 1*cm)
                paragraph.drawOn(self.canv, 0, -6*cm)
            finally:
                self.canv.restoreState()

    class RecipientInfo(Flowable):
        """Recipient info"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                session = self.parent.session
                translate = self.parent.translate
                now = tztime(datetime.now(timezone.utc))
                target = session.get_target()
                quotation_object = session.label
                if IWfCatalogEntry.providedBy(target):
                    data_type = target.get_data_type()
                    if data_type is not None:
                        quotation_object = II18n(data_type).query_attribute('label', request=self.request)
                formatter = '{} <font name="Roboto-Medium">{}</font>'
                header = [
                    formatter.format(translate(_('#Quotation:')),
                                     self.parent.booking.quotation_number),
                    formatter.format(translate(_("Date:")),
                                     format_date(now, SH_DATE_FORMAT)),
                    formatter.format(translate(_("Object:")),
                                     quotation_object or MISSING_INFO)
                ]
                paragraph = Paragraph(f"{'<br />'.join(header)}", STYLESHEET['p'])
                paragraph.wrap(8*cm, 4*cm)
                paragraph.drawOn(self.canv, 0*cm, -8*cm)
                recipient = get_principal(None, self.parent.booking.recipient)
                recipient_info = IUserProfile(recipient)
                recipient_address = recipient_info.establishment_address
                frame = [
                    recipient_info.establishment,
                    recipient_address.street,
                    recipient_address.locality,
                    f'{recipient_address.postal_code}  {recipient_address.city}'
                ]
                paragraph = Paragraph(f"{'<br />'.join(filter(bool, frame))}", STYLESHEET['frame'])
                paragraph.wrap(8*cm, 4*cm)
                height = paragraph.height
                paragraph.drawOn(self.canv, 8.5*cm, -7.85*cm - height/2)
                settings = IMovieTheaterSettings(self.parent.theater)
                self.canv.setStrokeColor(settings.get_quotation_color())
                self.canv.rect(8*cm, -9.5*cm, 9*cm, 3.25*cm, stroke=1, fill=0)
            finally:
                self.canv.restoreState()

    class QuotationDetail(Flowable):
        """Quotations details"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                booking = self.parent.booking
                session = self.parent.session
                settings = IMovieTheaterSettings(self.parent.theater)
                color = settings.get_quotation_color()
                translate = self.parent.translate
                style = TableStyle([
                    ('ALIGN', (1, 0), (-1, 0), 'CENTER'),
                    ('ALIGN', (1, 1), (1, -1), 'CENTER'),
                    ('ALIGN', (2, 1), (-1, -1), 'RIGHT'),
                    ('ALIGN', (-3, -3), (-2, -1), 'RIGHT'),
                    ('BOX', (0, 0), (-1, 0), 0.5, color),
                    ('BOX', (0, 0), (-1, -4), 0.5, color),
                    ('BOX', (-3, -3), (-1, -1), 0.5, color),
                    ('LINEBEFORE', (0, 0), (-1, -4), 0.5, color),
                    ('SPAN', (-3, -3), (-2, -3)),
                    ('SPAN', (-3, -2), (-2, -2)),
                    ('SPAN', (-3, -1), (-2, -1))
                ])
                preview_args = self.parent.kwargs
                price_id = preview_args.get('price')
                if price_id:
                    price = ICinemaPriceContainer(self.parent.theater).get(price_id)
                else:
                    price = booking.get_price()
                nb_participants = preview_args.get('nb_participants')
                if nb_participants is None:
                    nb_participants = booking.nb_participants
                else:
                    nb_participants = int(nb_participants or 0)
                nb_accompanists = preview_args.get('nb_accompanists')
                if nb_accompanists is None:
                    nb_accompanists = booking.nb_accompanists
                else:
                    nb_accompanists = int(nb_accompanists or 0)
                nb_free_accompanists = preview_args.get('nb_free_accompanists')
                if nb_free_accompanists is None:
                    nb_free_accompanists = booking.nb_free_accompanists
                else:
                    nb_free_accompanists = int(nb_free_accompanists or 0)
                nb_paid_accompanists = nb_accompanists - nb_free_accompanists
                if price is not None:
                    if price.accompanying_price == 0.:
                        free_accompanists = nb_accompanists
                        paid_accompanists = 0
                    else:
                        ratio = preview_args.get('ratio')
                        if ratio is None:
                            ratio = booking.accompanying_ratio
                        else:
                            ratio = float(ratio or 0)
                        if ratio:
                            max_accompanists = math.ceil(nb_participants / ratio)
                            free_accompanists = min(max_accompanists, nb_paid_accompanists) + nb_free_accompanists
                            paid_accompanists = nb_accompanists - free_accompanists
                        else:
                            free_accompanists = nb_free_accompanists
                            paid_accompanists = nb_paid_accompanists
                    participants_amount = price.participant_price * nb_participants
                    accompanying_amount = price.accompanying_price * paid_accompanists
                else:
                    free_accompanists = nb_accompanists
                    paid_accompanists = 0
                    participants_amount = 0.
                    accompanying_amount = 0.
                total_amount = round(participants_amount + accompanying_amount, 2)
                base_amount = round(total_amount / (1. + settings.vat_rate/100), 2)
                vat_amount = total_amount - base_amount
                data = [
                    (translate(_("Activity")),
                     translate(_("Units")),
                     translate(_("Unit price")),
                     translate(_("Total"))),
                    (f'{session.get_label()}, {format_datetime(session.start_date)}',
                     '',
                     '',
                     ''),
                    (translate(_("   - Entries ({})")).format(
                        translate(_("1 group")) if booking.nb_groups == 1
                        else translate(_("{} groups")).format(booking.nb_groups)),
                     booking.nb_participants,
                     f'{price.participant_price:.2f} {settings.currency}',
                     f'{participants_amount:.2f} {settings.currency}')
                ]
                if free_accompanists:
                    data.append(
                        (translate(_("   - Free accompanists")),
                         free_accompanists or '--',
                         f'{0:.2f} {settings.currency}',
                         f'{0:.2f} {settings.currency}')
                    )
                if paid_accompanists:
                    data.append(
                        (translate(_("   - Paying accompanists")),
                         paid_accompanists or '--',
                         f'{price.accompanying_price:.2f} {settings.currency}',
                         f'{accompanying_amount:.2f} {settings.currency}')
                    )
                message = booking.quotation_message
                if message:
                    data.append(('', '', '', ''))
                    data.append((Paragraph(message.replace('\r', '')), '', '', ''))
                data.extend([
                    ('',
                     translate(_("Base amount")),
                     '',
                     f'{base_amount:.2f} {settings.currency}'),
                    ('',
                     '{} {:.2f}%'.format(translate(_("VAT")), settings.vat_rate),
                     '',
                     f'{vat_amount:.2f} {settings.currency}'),
                    ('',
                     translate(_("Total amount")),
                     '',
                     f'{total_amount:.2f} {settings.currency}')
                ])
                table = Table(data,
                              colWidths=(10*cm, 2*cm, 2*cm, 3*cm),
                              style=style)
                table.wrap(17*cm, 20*cm)
                height = sum(table._rowHeights)
                table.drawOn(self.canv, 0*cm, -10*cm - height)
            finally:
                self.canv.restoreState()

    class BankInfo(Flowable):
        """Bank info"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                settings = self.parent.theater.banking_account
                translate = self.request.localizer.translate
                style = TableStyle([
                    ('FONTSIZE', (0, 0), (-1, -1), 9),
                    ('ALIGN', (0, 0), (-1, -1), 'CENTER'),
                    ('GRID', (0, 0), (-1, -1), 0.5, black),
                    ('SPAN', (0, 0), (-1, 0)),
                    ('SPAN', (0, -1), (1, -1)),
                    ('SPAN', (2, -1), (-1, -1))
                ])
                data = [
                    (translate(_("Bank location: {}")).format(settings.company_name),),
                    (translate(_("Bank code")),
                     translate(_("Counter code")),
                     translate(_("Account number")),
                     translate(_("Key"))),
                    (settings.bank_code,
                     settings.counter_code,
                     settings.account_number,
                     settings.account_key),
                    (translate(_("IBAN: {}")).format(
                        get_text_parts(settings.iban_number, 0, 4, 8, 12, 16, 20, 24, -1)),
                     None,
                     translate(_("BIC: {}")).format(settings.bic_code),
                     None)
                ]
                table = Table(data,
                              colWidths=(5*cm, 5*cm, 5*cm, 2*cm),
                              style=style)
                table.wrap(17*cm, 4*cm)
                table.drawOn(self.canv, 0*cm, -25*cm)
            finally:
                self.canv.restoreState()

    class GeneralTheaterInfo(Flowable):
        """General theater info"""

        def __init__(self, parent, request):
            super().__init__()
            self.parent = parent
            self.request = request

        def draw(self):
            self.canv.saveState()
            try:
                settings = IMovieTheaterSettings(self.parent.theater)
                admin_info = self.parent.theater.admin_info
                translate = self.request.localizer.translate
                style = TableStyle([
                    ('FONTSIZE', (0, 0), (-1, -1), 9),
                    ('ALIGN', (0, 0), (-1, 0), 'CENTER'),
                    ('LINEABOVE', (0, 0), (-1, 0), 1, settings.get_quotation_color())
                ])
                data = [
                    (translate(_("APE: {}")).format(admin_info.ape_code),
                     translate(_("SIRET: {}")).format(
                         get_text_parts(admin_info.siret_code, 0, 3, 6, 9, -1)),
                     translate(_("VAT: {}")).format(
                         get_text_parts(admin_info.vat_number, 0, 4, 7, 10, -1)))
                ]
                table = Table(data,
                              colWidths=(5*cm, 7*cm, 5*cm),
                              style=style)
                table.wrap(17*cm, 1*cm)
                table.drawOn(self.canv, 0*cm, -27*cm)
            finally:
                self.canv.restoreState()

    def __init__(self, booking, quotation_number, **kwargs):
        self.built = False
        self.booking = booking
        self.session = ISession(booking)
        self.theater = IMovieTheater(booking)
        self.kwargs = kwargs
        self.report = BytesIO()
        self.elements = []
        request = query_request()
        if request is not None:
            translate = request.localizer.translate
        else:
            translate = str
        self.translate = translate
        self.document = BaseDocTemplate(self.report,
                                        title=translate(_("Quotation: {}")).format(quotation_number),
                                        author=II18n(self.theater).query_attribute('title', request=request),
                                        subject=self.session.label,
                                        creator=request.principal.title,
                                        pagesize=A4,
                                        leftMargin=2.*cm,
                                        rightMargin=2.*cm,
                                        topMargin=2.*cm,
                                        bottomMargin=2.*cm,
                                        pageCompression=1)
        self.document.addPageTemplates(Quotation.Template(self))
        self.append(Quotation.Header(self, request))
        self.append(Quotation.SubHeader(self, request))
        self.append(Quotation.RecipientInfo(self, request))
        self.append(Quotation.QuotationDetail(self, request))
        self.append(Quotation.BankInfo(self, request))
        self.append(Quotation.GeneralTheaterInfo(self, request))
        self.build()

    def __bytes__(self):
        if self.built:
            return self.report.getvalue()
        return None

    def append(self, flowable: Flowable):
        self.elements.append(flowable)

    def build(self):
        self.document.build(self.elements)
        self.built = True



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime

from persistent import Persistent
from pyramid.interfaces import IRequest
from zope.container.contained import Contained
from zope.copy.interfaces import ICopyHook, ResumeCopy
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_app_msc.feature.booking import IBookingContainer
from pyams_app_msc.feature.planning.interfaces import IPlanning, IPlanningTarget, ISession
from pyams_app_msc.feature.profile import IOperatorProfile
from pyams_app_msc.feature.profile.interfaces import SEATS_DISPLAY_MODE
from pyams_app_msc.interfaces import MANAGE_PLANNING_PERMISSION, VIEW_PLANNING_PERMISSION
from pyams_app_msc.shared.theater import ICinemaRoomContainer, IMovieTheater
from pyams_app_msc.shared.theater.interfaces.room import ROOMS_VOCABULARY
from pyams_content.feature.filter.interfaces import IFilterValues
from pyams_content.interfaces import IObjectType
from pyams_content_api.feature.json import JSONBaseExporter
from pyams_content_api.feature.json.interfaces import IJSONExporter
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer, IPyAMSUserLayer
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.date import SH_TIME_FORMAT, format_date, format_time
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue, MISSING_INFO
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_workflow.interfaces import IWorkflowVersions
from pyams_zmi.interfaces import IObjectLabel, SMALL_TITLE_SPAN_BREAK
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(ISession)
class Session(Persistent, Contained):
    """Session persistent class"""

    label = FieldProperty(ISession['label'])
    _start_date = FieldProperty(ISession['start_date'])
    _duration = FieldProperty(ISession['duration'])
    _end_date = FieldProperty(ISession['end_date'])
    room = FieldProperty(ISession['room'])
    capacity = FieldProperty(ISession['capacity'])
    version = FieldProperty(ISession['version'])
    audiences = FieldProperty(ISession['audiences'])
    temporary = FieldProperty(ISession['temporary'])
    bookable = FieldProperty(ISession['bookable'])
    extern_bookable = FieldProperty(ISession['extern_bookable'])
    comments = FieldProperty(ISession['comments'])
    notepad = FieldProperty(ISession['notepad'])

    @property
    def start_date(self):
        return self._start_date

    @start_date.setter
    def start_date(self, value):
        self._start_date = tztime(value)

    @property
    def duration(self):
        return self._duration

    @duration.setter
    def duration(self, value):
        self._duration = value

    @property
    def end_date(self):
        return self._end_date

    @end_date.setter
    def end_date(self, value):
        self._end_date = tztime(value)
        self._duration = self._end_date - self._start_date

    def get_room(self):
        """Room getter"""
        parent = get_parent(self, IMovieTheater)
        return ICinemaRoomContainer(parent).get(self.room)

    def get_target(self):
        """Planning target getter"""
        target = get_parent(self, IPlanningTarget)
        versions = IWorkflowVersions(target, None)
        if versions is not None:
            target = versions.get_version(-1)
        return target

    def get_label(self):
        """Session label getter"""
        label = self.label
        if not label:
            target = self.get_target()
            if target is not None:
                label = II18n(target).query_attribute('title')
        return label or MISSING_INFO


@adapter_config(required=ISession,
                provides=IObjectType)
def session_object_type(context):
    """Session object type getter"""
    return get_interface_base_name(ISession)


@adapter_config(name='audiences',
                required=ISession,
                provides=IFilterValues)
def session_filter_values(context):
    """Session filter values adapter"""
    yield from (
        f'audience:{audience}'
        for audience in context.audiences or ()
    )


@adapter_config(required=ISession,
                provides=ICopyHook)
class SessionCopyHook(ContextAdapter):
    """Session copy hook"""

    def __call__(self, toplevel, register):
        register(self._clean_bookings)
        raise ResumeCopy

    def _clean_bookings(self, translate):
        session = translate(self.context)
        # clear clone session bookings
        bookings = IBookingContainer(session)
        for name in list(bookings.keys()):
            del bookings[name]


@adapter_config(required=(ISession, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def get_session_label(context, request, view, formatter='html'):
    """Session label getter"""
    translate = request.localizer.translate
    target = context.get_target()
    if IMovieTheater.providedBy(target):
        label = f'({context.label})' or translate(_("Out of catalog activity"))
    else:
        label = get_object_label(target, request, view)
    rooms = getVocabularyRegistry().get(context, ROOMS_VOCABULARY)
    return (SMALL_TITLE_SPAN_BREAK if formatter == 'html' else '{} - {}').format(
        label,
        translate(_("{room} - {date} at {start_time}")).format(
            room=rooms.by_value.get(context.room).title,
            date=format_date(context.start_date),
            start_time=format_time(context.start_date, SH_TIME_FORMAT)
        ))


@adapter_config(name='text',
                required=(ISession, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def get_session_text_label(context, request, view):
    """Session text label getter"""
    return get_session_label(context, request, view, 'text')


@adapter_config(required=(ISession, IPyAMSUserLayer, Interface),
                provides=IObjectLabel)
def get_session_user_label(context, request, view, formatter='html'):
    """Session user label getter"""
    translate = request.localizer.translate
    target = context.get_target()
    if IMovieTheater.providedBy(target):
        label = context.label or '--'
    else:
        label = get_object_label(target, request, view, formatter)
    return (SMALL_TITLE_SPAN_BREAK if formatter == 'html' else '{} - {}').format(
        label,
        translate(_("{date} at {start_time}")).format(
            date=format_date(context.start_date),
            start_time=format_time(context.start_date, SH_TIME_FORMAT)
        ))


@adapter_config(name='text',
                required=(ISession, IPyAMSUserLayer, Interface),
                provides=IObjectLabel)
def get_session_user_text_label(context, request, view):
    """Session user text label getter"""
    return get_session_user_label(context, request, view, 'text')


@adapter_config(name='short',
                required=(ISession, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def get_session_short_label(context, request, view, formatter='html'):
    """Session short label getter"""
    translate = request.localizer.translate
    target = get_parent(context, IPlanningTarget)
    versions = IWorkflowVersions(target, None)
    if versions is not None:
        target = versions.get_version(-1)
    return ('<small>{}</small><br />{}' if formatter == 'html' else '{} - {}').format(
        get_object_label(target, request, view),
        translate(_(" {date} at {start_time}")).format(
            date=format_date(context.start_date),
            start_time=format_time(context.start_date, SH_TIME_FORMAT)
        ))


@adapter_config(name='short-text',
                required=(ISession, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def get_session_short_text_label(context, request, view):
    """Session short text label getter"""
    return get_session_short_label(context, request, view, 'text')


@adapter_config(required=ISession,
                provides=IViewContextPermissionChecker)
class SessionPermissionChecker(ContextAdapter):
    """Session permission checker"""

    edit_permission = MANAGE_PLANNING_PERMISSION


@adapter_config(required=ISession,
                provides=IPlanning)
def session_planning(context):
    """Session planning adapter"""
    target = get_parent(context, IPlanningTarget)
    return IPlanning(target, None)


@adapter_config(required=ISession,
                provides=IMovieTheater)
def session_movie_theater(context):
    """Session movie theater adapter"""
    return get_parent(context, IMovieTheater)


@adapter_config(required=(ISession, IRequest),
                provides=IJSONExporter)
class JSONSessionExporter(JSONBaseExporter):
    """JSON session exporter"""

    def _add_seats(self, value):
        """Add capacity and confirmed seats to label"""
        if not self.context.bookable:
            return f'{value} \n'
        profile = IOperatorProfile(self.request)
        if profile.session_seats_display_mode == SEATS_DISPLAY_MODE.NONE.value:
            return f'{value} \n'
        else:
            container = IBookingContainer(self.context)
            return f'{value} \n({container.get_seats(profile.session_seats_display_mode)})'

    def convert_content(self, **params):
        result = super().convert_content(**params)
        context = self.context
        request = self.request
        result['id'] = ICacheKeyValue(context)
        result['href'] = absolute_url(context, request)
        target = get_parent(context, IPlanningTarget)
        if self.context.label:
            self.get_attribute(result, 'label', name='title', converter=self._add_seats)
        else:
            versions = IWorkflowVersions(target, None)
            if versions is not None:
                last_entry = versions.get_version()
                self.get_i18n_attribute(result, 'title', params.get('lang'),
                                        context=last_entry, converter=self._add_seats)
        self.get_attribute(result, 'start_date', 'start', converter=datetime.isoformat)
        self.get_attribute(result, 'end_date', 'end', converter=datetime.isoformat)
        self.get_attribute(result, 'room')
        self.get_attribute(result, 'capacity')
        self.get_attribute(result, 'temporary')
        self.get_attribute(result, 'bookable')
        self.get_attribute(result, 'notepad')
        if params.get('with_edit_info'):
            planning = IPlanning(target)
            edit_context = params.get('edit_context')
            if edit_context is None:
                edit_context = self.request.context
            editable = bool((planning is edit_context) and
                            request.has_permission(MANAGE_PLANNING_PERMISSION, context=planning))
            result['editable'] = editable
            result['visible'] = bool(request.has_permission(VIEW_PLANNING_PERMISSION, context=planning))
            if self.context.temporary:
                result['borderWidth'] = '3'
                result['borderStyle'] = 'dashed'
            if planning is edit_context:
                if not self.context.bookable:
                    if 'textColor' not in result:
                        result['textColor'] = 'var(--fc-nobookable-text)'
                        result['backgroundColor'] = 'var(--fc-nobookable-bg)'
                        result['borderColor'] = 'var(--fc-nobookable-border)'
                else:
                    if self.context.extern_bookable:
                        result['borderWidth'] = 3
                    container = IBookingContainer(context)
                    if container.free_seats < 0:
                        result['textColor'] = 'var(--fc-warning-text)'
                        result['backgroundColor'] = 'var(--fc-warning-bg)'
                        result['borderColor'] = 'var(--fc-warning-border)'
                    elif container.get_confirmed_seats() > 0:
                        result['textColor'] = 'var(--fc-accepted-text)'
                        result['backgroundColor'] = 'var(--fc-accepted-bg)'
                        result['borderColor'] = 'var(--fc-accepted-border)'
                    else:
                        result['textColor'] = 'var(--fc-enabled-text)'
                        result['backgroundColor'] = 'var(--fc-enabled-bg)'
                        result['borderColor'] = 'var(--fc-enabled-border)'
            else:
                result['textColor'] = 'var(--fc-disabled-text)'
                result['backgroundColor'] = 'var(--fc-disabled-bg)'
                result['borderColor'] = 'var(--fc-disabled-border)'
        return result



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le, Or
from persistent import Persistent
from pyramid.decorator import reify
from zope.container.contained import Contained
from zope.interface import Interface
from zope.intid.interfaces import IIntIds
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.booking import IBookingContainer
from pyams_app_msc.feature.planning.interfaces import ISession, VERSION_INFO, VERSION_INFO_ABBR
from pyams_app_msc.shared.catalog import ICatalogEntry, ICatalogEntryInfo, IWfCatalogEntry
from pyams_app_msc.shared.catalog.portlet.skin.interfaces import ICatalogEntrySpecificitiesPortletRendererSettings
from pyams_app_msc.shared.theater import IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.audience import get_audience_contact
from pyams_app_msc.shared.theater.interfaces import SESSION_REQUEST_MODE
from pyams_catalog.query import CatalogResultSet, IsNone
from pyams_content.shared.common.portlet.interfaces import ISharedContentSpecificitiesPortletSettings
from pyams_i18n.language import BASE_LANGUAGES
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config, get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import canonical_url

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@factory_config(ICatalogEntrySpecificitiesPortletRendererSettings)
class CatalogEntrySpecificitiesPortletRendererSettings(Persistent, Contained):
    """Catalog entry specificities portlet renderer settings"""

    display_sessions = FieldProperty(ICatalogEntrySpecificitiesPortletRendererSettings['display_sessions'])
    sessions_weeks = FieldProperty(ICatalogEntrySpecificitiesPortletRendererSettings['sessions_weeks'])
    display_free_seats = FieldProperty(ICatalogEntrySpecificitiesPortletRendererSettings['display_free_seats'])


@adapter_config(name='msc:catalog_entry',
                required=(IPortalContext, IPyAMSLayer, Interface, ISharedContentSpecificitiesPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/specificities.pt',
                 layer=IPyAMSLayer)
class CatalogEntrySpecificitiesPortletRenderer(PortletRenderer):
    """Catalog entry specificities portlet renderer"""

    label = _("MSC: catalog entry specificities with sessions")
    weight = 100

    settings_interface = ICatalogEntrySpecificitiesPortletRendererSettings

    use_authentication = True
    entry_info = None

    def update(self):
        super().update()
        self.entry_info = ICatalogEntryInfo(self.context, None)

    def render(self, template_name=''):
        if self.entry_info is None:
            return ''
        return super().render(template_name)

    @reify
    def audience(self):
        """Audience getter"""
        return self.request.params.get('audience')

    def get_language(self, value):
        translate = self.request.localizer.translate
        return translate(BASE_LANGUAGES.get(value, _("(unknown)")))

    def get_version(self, session):
        if not session.version:
            return None
        translate = self.request.localizer.translate
        return translate(VERSION_INFO_ABBR.get(VERSION_INFO(session.version), _("(undefined)")))

    def get_duration(self, value):
        translate = self.request.localizer.translate
        hours = value // 60
        minutes = value % 60
        if hours == 0:
            return translate(_("{} minutes")).format(minutes)
        return translate(_("{}h {}min")).format(hours, minutes)

    def get_sessions(self):
        """Display incoming sessions for provided catalog entry"""
        entry = get_parent(self.context, ICatalogEntry)
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        now = tztime(datetime.now(timezone.utc))
        params = [
            Eq(catalog['object_types'], get_interface_base_name(ISession)),
            Eq(catalog['parents'], intids.queryId(entry)),
            Ge(catalog['planning_start_date'], now),
            Le(catalog['planning_end_date'], now + timedelta(weeks=self.renderer_settings.sessions_weeks))
        ]
        if self.audience:
            params.append(Or(Eq(catalog['planning_audience'], self.audience),
                             IsNone(catalog['planning_audience'])))
        query = And(*params)
        yield from sorted(filter(lambda x: x.extern_bookable,
                                 CatalogResultSet(CatalogQuery(catalog).query(query))),
                          key=lambda x: x.start_date)

    @staticmethod
    def get_free_seats(session):
        """Get free seats for provided session"""
        bookings = IBookingContainer(session, None)
        if bookings is None:
            return session.capacity, session.capacity
        return session.capacity - bookings.get_confirmed_seats(), session.capacity

    def get_session_request_url(self, context):
        """Get URL for a new session request view"""
        if not IWfCatalogEntry(context).can_request_session():
            return None
        theater = get_parent(context, IMovieTheater)
        settings = IMovieTheaterSettings(theater)
        if settings.session_request_mode == SESSION_REQUEST_MODE.FORM.value:
            params = {}
            audiences = IWfCatalogEntry(self.context).audiences
            if len(audiences or ()) == 1:
                params['audience'] = audiences[0]
            elif self.audience:
                params['audience'] = self.audience
            return canonical_url(context, self.request, 'session-request.html', params)
        contact_email = None
        theater = get_parent(self.context, IMovieTheater)
        for audience_id in self.context.audiences:
            contact_email = get_audience_contact(theater, audience_id)
            if contact_email is not None:
                break
        return f'mailto:{contact_email}' if contact_email else None



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.feature.navigation.interfaces import INavigationViewletManager
from pyams_app_msc.skin import IPyAMSMSCLayer
from pyams_template.template import template_config
from pyams_viewlet.manager import TemplateBasedViewletManager, WeightOrderedViewletManager, viewletmanager_config
from pyams_viewlet.viewlet import Viewlet

__docformat__ = 'restructuredtext'


@viewletmanager_config(name='msc.navigation',
                       layer=IPyAMSMSCLayer,
                       provides=INavigationViewletManager)
@template_config(template='templates/navigation.pt',
                 layer=IPyAMSMSCLayer)
class NavigationViewletManager(TemplateBasedViewletManager, WeightOrderedViewletManager):
    """Navigation viewlet manager"""


@template_config(template='templates/navigation-menu.pt',
                 layer=IPyAMSMSCLayer)
class NavigationMenuItem(Viewlet):
    """Navigation menu"""

    css_class = ''



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.events import subscriber
from pyramid.httpexceptions import HTTPBadRequest
from zope.annotation.interfaces import IAnnotations
from zope.interface import Interface, Invalid, alsoProvides
from zope.schema import TextLine

from pyams_app_msc.feature.profile.interfaces import ActivatedPrincipalEvent, IUserProfile, \
    RegisteredPrincipalEvent, USER_PROFILE_KEY
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import IPrincipalInfo, PUBLIC_PERMISSION
from pyams_security.interfaces.plugin import ILocalUser, \
    IUserRegistrationConfirmationInfo as IUserRegistrationConfirmationInfoBase, LOCKED_ACCOUNT_PASSWORD
from pyams_security.interfaces.profile import IUserRegistrationViews
from pyams_security_views.interfaces.login import ILoginConfiguration
from pyams_skin.interfaces.viewlet import IFormHeaderViewletManager
from pyams_skin.schema.button import ResetButton, SubmitButton
from pyams_skin.viewlet.help import AlertMessage
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestAdapter, ContextRequestViewAdapter, adapter_config
from pyams_utils.factory import create_object
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@adapter_config(required=(Interface, IPyAMSLayer),
                provides=IUserRegistrationViews)
class UserRegistrationViews(ContextRequestAdapter):
    """User registration view getter"""

    @property
    def register_view(self):
        return absolute_url(self.context, self.request, 'register.html')

    @property
    def register_ok_view(self):
        return absolute_url(self.context, self.request, 'register-ok.html')

    @property
    def register_confirm_view(self):
        return absolute_url(self.context, self.request, 'register-confirm.html')

    @property
    def register_final_view(self):
        return absolute_url(self.context, self.request, 'register-final.html')

    @property
    def password_reset_view(self):
        return absolute_url(self.context, self.request, 'password-reset.html')

    @property
    def password_reset_final_view(self):
        return absolute_url(self.context, self.request, 'password-final.html')

    @property
    def password_change_view(self):
        return absolute_url(self.context, self.request, 'password-change.html')

    @property
    def password_change_final_view(self):
        return absolute_url(self.context, self.request, 'password-changed.html')


class IRegisterFormButtons(Interface):
    """Register form buttons interface"""

    register = SubmitButton(name='register',
                            title=_("Create account"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


@ajax_form_config(name='register.html',
                  layer=IPyAMSLayer)
class RegisterForm(AddForm, PortalContextIndexPage):
    """Register form"""

    title = _("Registration form")
    legend = _("Account settings")

    fields = Fields(IUserProfile).omit('principal_id', 'active')
    content_factory = IUserProfile

    buttons = Buttons(IRegisterFormButtons)

    _edit_permission = PUBLIC_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        email = self.widgets.get('email')
        if email is not None:
            email.object_data = {
                'input-mask': {
                    'mask': '*{1,64}[.*{1,64}][.*{1,64}][.*{1,63}]@*{1,63}.*{1,63}[.*{1,63}][.*{1,63}]'
                }
            }
            alsoProvides(email, IObjectData)
        phone_number = self.widgets.get('phone_number')
        if phone_number is not None:
            phone_number.object_data = {
                'input-mask': {
                    'mask': '[+9{3}] 99 99 99 99 99'
                }
            }
            alsoProvides(phone_number, IObjectData)
        structure_type = self.widgets.get('structure_type')
        if structure_type is not None:
            structure_type.prompt = True
            structure_type.prompt_message = _("Please select your structure type...")

    @handler(buttons['register'])
    def handle_register(self, action):
        self.handle_add(self, action)

    def add(self, obj):
        login_config = ILoginConfiguration(self.request.root)
        if not login_config.open_registration:
            raise HTTPBadRequest(_("Registration is actually closed!"))
        sm = get_utility(ISecurityManager)
        users_folder = sm.get(login_config.users_folder)
        if users_folder is not None:
            local_user = create_object(ILocalUser)
            local_user.login = obj.email
            local_user.email = obj.email
            local_user.firstname = obj.firstname
            local_user.lastname = obj.lastname
            local_user.company_name = obj.establishment
            local_user.password = LOCKED_ACCOUNT_PASSWORD
            local_user.generate_secret(notify=True, request=self.request)
            users_folder[local_user.login] = local_user
            principal = IPrincipalInfo(local_user)
            annotations = IAnnotations(principal)
            annotations[USER_PROFILE_KEY] = obj
            self.request.registry.notify(RegisteredPrincipalEvent(principal))


@subscriber(IDataExtractedEvent, form_selector=RegisterForm)
def extract_register_form_data(event: IDataExtractedEvent):
    """Extract register form data"""
    data = event.data
    email = data.get('email')
    if email:
        login_config = ILoginConfiguration(event.form.request.root)
        sm = get_utility(ISecurityManager)
        if login_config.open_registration:
            users_folder = sm.get(login_config.users_folder)
            if users_folder is not None:
                user = users_folder.get(email)
                if user is not None:
                    event.form.widgets.errors += (Invalid(_("This email address is already "
                                                            "registered!")),)


@viewlet_config(name='form-help',
                context=Interface, layer=IPyAMSLayer, view=RegisterForm,
                manager=IFormHeaderViewletManager, weight=10)
class RegisterFormHelp(AlertMessage):
    """Register form help"""

    status = 'info'

    _message = _("This form allows you to register a new user account.\n"
                 "After submitting the form, you will receive an email containing a link which "
                 "must be used to confirm your email address and set a new password.")


@adapter_config(required=(Interface, IPyAMSLayer, RegisterForm),
                provides=IAJAXFormRenderer)
class RegisterFormRenderer(ContextRequestViewAdapter):
    """Register form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        register_views = self.request.registry.getMultiAdapter((self.context, self.request),
                                                               IUserRegistrationViews)
        return {
            'status': 'redirect',
            'location': register_views.register_ok_view
        }


@pagelet_config(name='register-ok.html',
                layer=IPyAMSLayer)
@template_config(template='templates/register-ok.pt',
                 layer=IPyAMSLayer)
class RegisterOKView(PortalContextIndexPage):
    """Saved registration view"""

    @property
    def activation_delay(self):
        configuration = ILoginConfiguration(self.request.root)
        return configuration.activation_delay


class IUserRegistrationConfirmationInfo(IUserRegistrationConfirmationInfoBase):
    """User registration confirmation info interface"""

    login = TextLine(title=_("E-mail address"),
                     required=True)


class IUserRegistrationConfirmationButtons(Interface):
    """User registration confirmation buttons interface"""

    validate = SubmitButton(name='validate',
                            title=_("Validate account"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


@ajax_form_config(name='register-confirm.html',
                  layer=IPyAMSLayer)
class RegistrationConfirmForm(AddForm, PortalContextIndexPage):
    """Registration confirmation form"""

    title = _("Registration confirm form")
    legend = _("Confirm your login and set a new password")

    fields = Fields(IUserRegistrationConfirmationInfo).select('login', 'password', 'confirmed_password',
                                                              'activation_hash')
    buttons = Buttons(IUserRegistrationConfirmationButtons)

    _edit_permission = PUBLIC_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        hash = self.widgets.get('activation_hash')
        if hash is not None:
            hash.mode = HIDDEN_MODE
            hash.value = self.request.params.get('hash')

    @handler(buttons['validate'])
    def handle_validate(self, action):
        self.handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        login_config = ILoginConfiguration(self.request.root)
        sm = get_utility(ISecurityManager)
        users_folder = sm.get(login_config.users_folder)
        if users_folder is not None:
            user = users_folder.get(data.get('login'))
            if user is None:
                self.widgets.errors += (Invalid(_("Can't activate account with provided arguments!")),)
            else:
                hash = data.get('activation_hash')
                try:
                    user.check_activation(hash, user.login, LOCKED_ACCOUNT_PASSWORD)
                except Invalid as exc:
                    translate = self.request.localizer.translate
                    self.widgets.errors += (Invalid(', '.join(map(translate, exc.args))),)
                else:
                    user.password = data.get('password')
                    principal = IPrincipalInfo(user)
                    self.request.registry.notify(ActivatedPrincipalEvent(principal))
                    return user


@adapter_config(required=(Interface, IPyAMSLayer, RegistrationConfirmForm),
                provides=IAJAXFormRenderer)
class RegistrationConfirmFormRenderer(ContextRequestViewAdapter):
    """Registration confirm form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        register_views = self.request.registry.getMultiAdapter((self.context, self.request),
                                                               IUserRegistrationViews)
        return {
            'status': 'redirect',
            'location': register_views.register_final_view
        }


@pagelet_config(name='register-final.html',
                layer=IPyAMSLayer)
@template_config(template='templates/register-final.pt',
                 layer=IPyAMSLayer)
class RegistrationFinalView(PortalContextIndexPage):
    """Registration final view"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.profile.interfaces module

This module defines user profile interfaces.
"""

from enum import Enum

from zope.interface import Attribute, Interface, implementer
from zope.interface.interfaces import IObjectEvent, ObjectEvent
from zope.schema import Bool, Choice, List, Object, TextLine
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.component.address.interfaces import IAddress
from pyams_app_msc.reference.structure import STRUCTURE_TYPES_VOCABULARY
from pyams_app_msc.shared.theater.interfaces import MSC_THEATERS_VOCABULARY
from pyams_scheduler.interfaces import ITask
from pyams_utils.schema import MailAddressField

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


USER_PROFILE_KEY = 'msc.profile'


class IUserProfile(Interface):
    """User profile interface"""

    principal_id = TextLine(title="Principal ID",
                            description=_("Principal ID"),
                            required=True)

    active = Bool(title=_("Active user?"),
                  description=_("Inactive users can't login anymore to website"),
                  required=True,
                  default=True)

    firstname = TextLine(title=_("First name"),
                         required=True)

    lastname = TextLine(title=_("Last name"),
                        required=True)

    email = MailAddressField(title=_("Mail address"),
                             required=True)

    phone_number = TextLine(title=_("Phone number"),
                            required=True)

    establishment = TextLine(title=_("Establishment of affiliation"),
                             required=True)

    structure_type = Choice(title=_("Structure type"),
                            description=_("Select structure type matching this establishment"),
                            vocabulary=STRUCTURE_TYPES_VOCABULARY,
                            required=True)

    establishment_address = Object(title=_("Establishment address"),
                                   schema=IAddress,
                                   required=False)

    local_theaters = List(title=_("Preferred local theaters"),
                          value_type=Choice(vocabulary=MSC_THEATERS_VOCABULARY),
                          required=True)

    def get_structure_type(self):
        """Structure type label getter"""


class SEATS_DISPLAY_MODE(Enum):
    """Session seats display mode"""
    NONE = 'none'
    TOTAL = 'total'
    WAITING = 'waiting'
    CONFIRMED = 'confirmed'


SEATS_DISPLAY_LABELS = {
    SEATS_DISPLAY_MODE.NONE.value: _("Not displayed"),
    SEATS_DISPLAY_MODE.TOTAL.value: _("Display confirmed/requested/capacity seats"),
    SEATS_DISPLAY_MODE.WAITING.value: _("Display confirmed/waiting/capacity seats"),
    SEATS_DISPLAY_MODE.CONFIRMED.value: _("Display confirmed/capacity seats")
}


SEATS_DISPLAY_VOCABULARY = SimpleVocabulary([
    SimpleTerm(i, title=t)
    for i, t in SEATS_DISPLAY_LABELS.items()
])


OPERATOR_PROFILE_KEY = 'msc.operator'


class IOperatorProfile(Interface):
    """Operator management operator profile"""

    session_seats_display_mode = Choice(title=_("Session seats display mode"),
                                        description=_("You can choose how session seats are displayed in "
                                                      "movie theater planning view"),
                                        vocabulary=SEATS_DISPLAY_VOCABULARY,
                                        required=True,
                                        default=SEATS_DISPLAY_MODE.TOTAL.value)


class IRegisteredPrincipalEvent(IObjectEvent):
    """Registered principal event interface"""


@implementer(IRegisteredPrincipalEvent)
class RegisteredPrincipalEvent(ObjectEvent):
    """Registered principal event"""


class IActivatedPrincipalEvent(IObjectEvent):
    """Activated principal event interface"""


@implementer(IActivatedPrincipalEvent)
class ActivatedPrincipalEvent(ObjectEvent):
    """Activated principal event"""


class IUserProfileCleanerTask(ITask):
    """User profile cleaner task interface"""

    user_id = Attribute("User profile ID")



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.theater import IMovieTheater
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common.interfaces.types import IDataType, ITypedSharedTool
from pyams_content.shared.common.zmi.types import DataTypeAddForm, DataTypeEditForm, DataTypeLabelsGroup, \
    DataTypePictogramsGroup, \
    DataTypesAddAction
from pyams_content.shared.common.zmi.types.container import SharedToolTypesMenu, SharedToolTypesView
from pyams_content.shared.common.zmi.types.interfaces import ISharedToolTypesTable
from pyams_form.ajax import ajax_form_config
from pyams_form.interfaces.form import IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu, IToolbarViewletManager

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='data-types.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=405,
                permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterSharedToolTypesMenu(SharedToolTypesMenu):
    """Shared tool data types menu"""

    label = _("Activity types")


@viewlet_config(name='add-data-type.action',
                context=IMovieTheater, layer=IAdminLayer, view=ISharedToolTypesTable,
                manager=IToolbarViewletManager, weight=20,
                permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterDataTypesAddAction(DataTypesAddAction):
    """Data type add action"""

    label = _("Add activity type")


@ajax_form_config(name='add-data-type.html',
                  context=IMovieTheater, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterDataTypeAddForm(DataTypeAddForm):
    """Movie theater data type add form"""

    subtitle = _("New activity type")

    @property
    def fields(self):
        return super().fields.omit('source_folder', 'navigation_label',
                                   'facets_label', 'facets_type_label',
                                   'dashboard_label', 'color', 'pictogram')


@pagelet_config(name='data-types.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterSharedToolTypesView(SharedToolTypesView):
    """Movie theater data types view"""

    title = _("Activities types")
    table_label = _("Movie theater activities types list")


@ajax_form_config(name='properties.html',
                  context=IDataType, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
class MovieTheaterDataTypeEditForm(DataTypeEditForm):
    """Movie theater data type properties edit form"""

    @property
    def legend(self):
        manager = get_parent(self.context, IMovieTheater)
        if manager is not None:
            return _("Activity type properties")
        return super().legend

    @property
    def fields(self):
        fields = super().fields
        manager = get_parent(self.context, IMovieTheater)
        if manager is not None:
            fields = fields.omit('source_folder', 'navigation_label',
                                 'facets_label', 'facets_type_label', 'dashboard_label',
                                 'color', 'pictogram')
        return fields


@adapter_config(name='labels.group',
                required=(ITypedSharedTool, IAdminLayer, DataTypeAddForm),
                provides=IGroup)
@adapter_config(name='labels.group',
                required=(IDataType, IAdminLayer, DataTypeEditForm),
                provides=IGroup)
class MovieTheaterDataTypeLabelsGroup(DataTypeLabelsGroup):
    """Movie theater data type pictograms group"""

    def __new__(cls, context, request, view):
        manager = get_parent(context, IMovieTheater)
        if manager is not None:
            return None
        return DataTypeLabelsGroup.__new__(cls)


@adapter_config(name='pictograms.group',
                required=(ITypedSharedTool, IAdminLayer, DataTypeAddForm),
                provides=IGroup)
@adapter_config(name='pictograms.group',
                required=(IDataType, IAdminLayer, DataTypeEditForm),
                provides=IGroup)
class MovieTheaterDataTypePictogramsGroup(DataTypePictogramsGroup):
    """Movie theater data type pictograms group"""

    def __new__(cls, context, request, view):
        manager = get_parent(context, IMovieTheater)
        if manager is not None:
            return None
        return DataTypePictogramsGroup.__new__(cls)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from pyramid.authorization import ALL_PERMISSIONS, Allow
from pyramid.interfaces import IRequest
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.profile.interfaces import IOperatorProfile, IUserProfile, OPERATOR_PROFILE_KEY, \
    USER_PROFILE_KEY
from pyams_app_msc.reference.structure import IStructureTypeTable
from pyams_i18n.interfaces import II18n
from pyams_security.interfaces.base import IPrincipalInfo
from pyams_security.interfaces.names import ADMIN_USER_ID
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.registry import get_utility

__docformat__ = 'restructuredtext'


@factory_config(IUserProfile)
class UserProfile(Persistent, Contained):
    """User profile persistent class"""

    principal_id = None

    active = FieldProperty(IUserProfile['active'])
    firstname = FieldProperty(IUserProfile['firstname'])
    lastname = FieldProperty(IUserProfile['lastname'])
    email = FieldProperty(IUserProfile['email'])
    phone_number = FieldProperty(IUserProfile['phone_number'])
    establishment = FieldProperty(IUserProfile['establishment'])
    structure_type = FieldProperty(IUserProfile['structure_type'])
    establishment_address = FieldProperty(IUserProfile['establishment_address'])
    local_theaters = FieldProperty(IUserProfile['local_theaters'])

    def __acl__(self):
        return [
            (Allow, ADMIN_USER_ID, ALL_PERMISSIONS),
            (Allow, self.principal_id, ALL_PERMISSIONS)
        ]

    def get_structure_type(self):
        structures = get_utility(IStructureTypeTable)
        structure_type = structures.get(self.structure_type)
        if structure_type is None:
            return MISSING_INFO
        return II18n(structure_type).query_attribute('title')


@adapter_config(required=IPrincipalInfo,
                provides=IUserProfile)
def principal_user_profile(principal):
    """Principal user profile factory"""

    def user_profile_callback(profile):
        profile.principal_id = principal.id

    return get_annotation_adapter(principal, USER_PROFILE_KEY, IUserProfile,
                                  locate=False, callback=user_profile_callback)


@adapter_config(required=IRequest,
                provides=IUserProfile)
def request_user_profile(request):
    """Request user profile factory"""
    return IUserProfile(request.principal, None)


@factory_config(IOperatorProfile)
class OperatorProfile(Persistent, Contained):
    """Operator profile persistent class"""

    principal_id = None

    session_seats_display_mode = FieldProperty(IOperatorProfile['session_seats_display_mode'])


@adapter_config(required=IPrincipalInfo,
                provides=IOperatorProfile)
def principal_operator_profile(principal):
    """Principal operator profile factory"""

    def operator_profile_callback(profile):
        profile.principal_id = principal.id

    return get_annotation_adapter(principal, OPERATOR_PROFILE_KEY, IOperatorProfile,
                                  locate=False, callback=operator_profile_callback)


@adapter_config(required=IRequest,
                provides=IOperatorProfile)
def request_operator_profile(request):
    """Request operator profile factory"""
    return IOperatorProfile(request.principal, None)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.planning.interfaces import IMovieTheaterSession
from pyams_app_msc.feature.planning.session import Session
from pyams_utils.factory import factory_config


@factory_config(IMovieTheaterSession)
class MovieTheaterSession(Session):
    """Movie theater session persistent class"""

    activity = FieldProperty(IMovieTheaterSession['activity'])



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import datetime, timedelta, timezone

from persistent import Persistent
from pyramid.authorization import Allow, ALL_PERMISSIONS
from pyramid.events import subscriber
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS, IBookingContainer, IBookingInfo, \
    IBookingReminderTask, IBookingTarget
from pyams_app_msc.feature.messaging.interfaces import IMessagingSettings
from pyams_app_msc.feature.planning.interfaces import IPlanning, ISession
from pyams_app_msc.feature.profile.interfaces import IUserProfile
from pyams_app_msc.feature.quotation import Quotation
from pyams_app_msc.interfaces import CANCEL_BOOKING_PERMISSION, MANAGE_BOOKING_PERMISSION
from pyams_app_msc.shared.theater.interfaces import BOOKING_CANCEL_MODE, IMovieTheater, IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces.price import ICinemaPriceContainer
from pyams_catalog.utils import index_object
from pyams_content.feature.history.interfaces import IHistoryTarget
from pyams_content.interfaces import IObjectType
from pyams_file.property import FileProperty
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_mail.message import HTMLMessage
from pyams_scheduler.interfaces import IScheduler
from pyams_scheduler.interfaces.task import IDateTaskScheduling, SCHEDULER_TASK_DATE_MODE
from pyams_security.interfaces import ISecurityManager, IViewContextPermissionChecker
from pyams_security.interfaces.base import FORBIDDEN_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_ID
from pyams_security.security import ProtectedObjectMixin
from pyams_security.utility import get_principal
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import create_object, factory_config, get_interface_base_name
from pyams_utils.interfaces import ICacheKeyValue
from pyams_utils.registry import get_pyramid_registry, get_utility, query_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import load_object, volatile_property

__docformat__ = 'restructuredtext'


@factory_config(IBookingInfo)
@implementer(IHistoryTarget)
class BookingInfo(ProtectedObjectMixin, Persistent, Contained):
    """Booking information"""

    creator = FieldProperty(IBookingInfo['creator'])
    recipient = FieldProperty(IBookingInfo['recipient'])
    status = FieldProperty(IBookingInfo['status'])
    nb_participants = FieldProperty(IBookingInfo['nb_participants'])
    nb_accompanists = FieldProperty(IBookingInfo['nb_accompanists'])
    nb_free_accompanists = FieldProperty(IBookingInfo['nb_free_accompanists'])
    accompanying_ratio = FieldProperty(IBookingInfo['accompanying_ratio'])
    nb_groups = FieldProperty(IBookingInfo['nb_groups'])
    price = FieldProperty(IBookingInfo['price'])
    cultural_pass = FieldProperty(IBookingInfo['cultural_pass'])
    comments = FieldProperty(IBookingInfo['comments'])
    notepad = FieldProperty(IBookingInfo['notepad'])
    archived = FieldProperty(IBookingInfo['archived'])
    quotation_number = FieldProperty(IBookingInfo['quotation_number'])
    quotation_message = FieldProperty(IBookingInfo['quotation_message'])
    quotation = FileProperty(IBookingInfo['quotation'])
    send_update = FieldProperty(IBookingInfo['send_update'])
    update_subject = FieldProperty(IBookingInfo['update_subject'])
    update_message = FieldProperty(IBookingInfo['update_message'])
    send_reminder = FieldProperty(IBookingInfo['send_reminder'])
    reminder_subject = FieldProperty(IBookingInfo['reminder_subject'])
    reminder_message = FieldProperty(IBookingInfo['reminder_message'])
    reminder_date = FieldProperty(IBookingInfo['reminder_date'])

    def __acl__(self):
        result = [
            (Allow, ADMIN_USER_ID, ALL_PERMISSIONS)
        ]
        theater = get_parent(self, IMovieTheater)
        settings = IMovieTheaterSettings(theater)
        if settings.booking_cancel_mode != BOOKING_CANCEL_MODE.FORBIDDEN.value:
            result.append((Allow, self.recipient, CANCEL_BOOKING_PERMISSION))
        return result

    def get_recipient(self, with_info=True):
        """Recipient principal getter"""
        principal = get_principal(None, principal_id=self.recipient)
        if with_info:
            profile = IUserProfile(principal, None)
            return principal, profile
        return principal

    @property
    def nb_seats(self):
        """Total seats count"""
        return (self.nb_participants or 0) + (self.nb_accompanists or 0)

    def get_price(self):
        """Price getter"""
        if not self.price:
            return None
        theater = IMovieTheater(self)
        return ICinemaPriceContainer(theater).get(self.price)

    @volatile_property
    def session(self):
        """Booking session getter"""
        return get_parent(self, IBookingTarget)

    @property
    def session_index(self):
        """Booking session index value getter"""
        return ICacheKeyValue(self.session)

    def set_session(self, new_session):
        """Update booking session"""
        if not ISession.providedBy(new_session):
            new_session = load_object(new_session, self)
        if (new_session is self.session) or not ISession.providedBy(new_session):
            return
        old_bookings = self.__parent__
        new_bookings = IBookingContainer(new_session)
        new_bookings.append(self)
        index_object(self)
        del self.session
        if self.quotation is not None:
            del self.quotation
            self.get_quotation(force_refresh=True)
        del old_bookings[ICacheKeyValue(self)]
        registry = get_pyramid_registry()
        registry.notify(ObjectModifiedEvent(self))
        return self

    def get_quotation_number(self):
        theater = IMovieTheater(self)
        settings = IMovieTheaterSettings(theater)
        return settings.get_quotation_number()

    def get_quotation(self, force_refresh=False, store=True, **kwargs):
        quotation = self.quotation
        if force_refresh or (not quotation):
            quotation_number = self.quotation_number
            if not quotation_number:
                quotation_number = self.quotation_number = self.get_quotation_number()
            quotation = (f'{quotation_number}.pdf',
                         bytes(Quotation(self, quotation_number, **kwargs)))
            if store:
                self.quotation = quotation
        return quotation

    def send_reminder_message(self):
        """Send reminder message"""
        if not (self.send_reminder and self.reminder_message):
            return
        root = get_parent(self, ISiteRoot)
        settings = IMessagingSettings(root, None)
        if settings is None:
            return
        mailer = settings.get_mailer()
        if mailer is None:
            return
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(self.recipient)
        mail_info = IPrincipalMailInfo(principal, None)
        if mail_info is None:
            return
        mail_addresses = [
            f'{name} <{address}>'
            for name, address in mail_info.get_addresses()
        ]
        message = HTMLMessage(f'{settings.subject_prefix} {self.reminder_subject}',
                              from_addr=f'{settings.source_name} <{settings.source_address}>',
                              to_addr=mail_addresses,
                              html=self.reminder_message)
        mailer.send(message)
        self.reminder_date = tztime(datetime.now(timezone.utc))


@adapter_config(required=IBookingInfo,
                provides=IObjectType)
def booking_object_type(context):
    """Booking object type value adapter"""
    return get_interface_base_name(IBookingInfo)


@adapter_config(required=IBookingInfo,
                provides=IViewContextPermissionChecker)
class BookingInfoPermissionChecker(ContextAdapter):
    """Booking permission checker"""

    @property
    def edit_permission(self):
        if self.context.archived:
            return FORBIDDEN_PERMISSION
        return MANAGE_BOOKING_PERMISSION


@adapter_config(name='delete',
                required=IBookingInfo,
                provides=IViewContextPermissionChecker)
class BookingInfoDeletePermissionChecker(ContextAdapter):
    """Booking delete permission checker"""

    @property
    def edit_permission(self):
        if self.context.archived or (self.context.status == BOOKING_STATUS.ACCEPTED.value):
            return FORBIDDEN_PERMISSION
        return MANAGE_BOOKING_PERMISSION


@adapter_config(required=IBookingInfo,
                provides=IBookingContainer)
def booking_container_adapter(context):
    """Booking container adapter"""
    return context.__parent__


@adapter_config(required=IBookingInfo,
                provides=ISession)
def booking_session_adapter(context):
    """Booking session adapter"""
    return context.session


@adapter_config(required=IBookingInfo,
                provides=IPlanning)
def booking_planning_adapter(context):
    """Booking planning adapter"""
    return get_parent(context, IPlanning)


@adapter_config(required=IBookingInfo,
                provides=IMovieTheater)
def booking_theater_adapter(context):
    """Booking theater adapter"""
    return get_parent(context, IMovieTheater)


@subscriber(IObjectModifiedEvent, context_selector=IBookingInfo)
def handle_modified_booking(event):
    """Handle modified booking"""
    scheduler = query_utility(IScheduler)
    if scheduler is None:
        return
    booking = IBookingInfo(event.object)
    theater = IMovieTheater(booking)
    task_id = f'reminder::{ICacheKeyValue(booking)}'
    if task_id in scheduler:
        del scheduler[task_id]
    settings = IMovieTheaterSettings(theater)
    if not settings.reminder_delay:
        return
    reminder_date = booking.session.start_date - timedelta(days=settings.reminder_delay)
    if reminder_date < tztime(datetime.now(timezone.utc)):
        return
    if booking.send_reminder and not booking.reminder_date:
        task = create_object(IBookingReminderTask, booking=booking)
        task.name = f'Booking {task_id}'
        task.schedule_mode = SCHEDULER_TASK_DATE_MODE
        scheduler_info = IDateTaskScheduling(task)
        scheduler_info.start_date = reminder_date
        scheduler_info.active = True
        scheduler[task_id] = task



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from hypatia.interfaces import ICatalog
from zope.annotation.interfaces import IAnnotations

from pyams_app_msc.shared.catalog import ICatalogEntryInfo, IWfCatalogEntry
from pyams_catalog.index import KeywordIndexWithInterface
from pyams_utils.finder import find_objects_providing
from pyams_utils.registry import get_local_registry, get_utility, set_local_registry

__docformat__ = 'restructuredtext'


def color_print(text, color='91'):
    """Print to console with color"""
    print(f'\033[{color}m{text}\033[00m')


def evolve(site):
    """Switch audiences from CatalogEntryInfo to IWfCatalogEntry"""
    old_registry = get_local_registry()
    try:
        registry = site.getSiteManager()
        set_local_registry(registry)
        # Replace catalog index
        catalog = get_utility(ICatalog)
        if 'catalog_audience' in catalog:
            del catalog['catalog_audience']
        catalog['catalog_audience'] = KeywordIndexWithInterface(IWfCatalogEntry,
                                                                discriminator='audiences')
        # Update all catalog entries with new audiences value
        updated = False
        for object in find_objects_providing(site, IWfCatalogEntry):
            catalog_entry = IWfCatalogEntry(object)
            # remove old annotations
            annotations = IAnnotations(catalog_entry)
            if 'msc.activity_info' in annotations:
                del annotations['msc.activity_info']
            # update audiences
            entry_info = ICatalogEntryInfo(catalog_entry)
            if hasattr(entry_info, 'audiences'):
                print(f"Updating audiences for catalog entry « {catalog_entry} »")
                catalog_entry.audiences = entry_info.audiences
                del entry_info.audiences
                updated = True
        if updated:
            color_print("Catalog has been updated; you should reindex the whole site using 'pyams_index' "
                        "and 'pyams_es_index' command-line scripts!")
    finally:
        set_local_registry(old_registry)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION
from pyams_content.reference.zmi.viewlet import ReferencesTablesMenu
from pyams_content.zmi.viewlet.toplinks import TopTabsViewletManager
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer


@viewletmanager_config(name='reference-tables.menu',
                       context=Interface, layer=IAdminLayer,
                       manager=TopTabsViewletManager, weight=40)
class MSCReferencesTablesMenu(ReferencesTablesMenu):
    """MSC reference tables menu"""

    menus_permissions = {MANAGE_REFERENCE_TABLE_PERMISSION}



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

import json
from datetime import datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Le
from pyramid.decorator import reify
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound
from pyramid.view import view_config

from pyams_app_msc.feature.planning import IPlanning, IWfPlanningTarget
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.interfaces import MANAGE_PLANNING_PERMISSION, VIEW_BOOKING_PERMISSION, \
    VIEW_PLANNING_PERMISSION
from pyams_app_msc.shared.theater import IMovieTheaterSettings
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_app_msc.shared.theater.interfaces.room import ICinemaRoomContainer
from pyams_catalog.query import CatalogResultSet
from pyams_content_api.feature.json.interfaces import IJSONExporter
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_skin.viewlet.actions import JsContextAction
from pyams_template.template import template_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, ISecondaryActionsViewletManager
from pyams_zmi.view import InnerAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='planning.menu',
                context=IWfPlanningTarget, layer=IAdminLayer,
                manager=IContentManagementMenu, weight=20,
                permission=VIEW_PLANNING_PERMISSION)
class PlanningMenu(NavigationMenuItem):
    """Planning menu"""

    label = _("Planning")
    icon_class = 'fas fa-calendar-week'
    href = '#planning.html'


@pagelet_config(name='planning.html',
                context=IWfPlanningTarget, layer=IPyAMSLayer,
                permission=VIEW_PLANNING_PERMISSION)
@template_config(template='templates/planning.pt', layer=IPyAMSLayer)
class PlanningView(InnerAdminView):
    """Planning view"""

    title = _("Sessions planning")

    @reify
    def theater(self):
        """Theater getter"""
        return get_parent(self.context, IMovieTheater)

    @property
    def rooms(self):
        """Theater rooms iterator getter"""
        theater = self.theater
        if theater is not None:
            yield from ICinemaRoomContainer(theater).get_active_items()

    def get_context(self):
        """View context getter"""
        return IPlanning(self.context)

    @property
    def can_view_bookings(self):
        """Bookings access checker"""
        return bool(self.request.has_permission(VIEW_BOOKING_PERMISSION, context=self.context))

    @property
    def can_edit_planning(self):
        """Calendar editor checker"""
        return bool(self.request.has_permission(MANAGE_PLANNING_PERMISSION, context=self.context))

    def get_calendar_options(self, room):
        """Calendar options getter"""
        context = self.get_context()
        request = self.request
        settings = IMovieTheaterSettings(self.theater)
        can_edit = request.has_permission(MANAGE_PLANNING_PERMISSION, context=context)
        now = tztime(datetime.now(timezone.utc))
        week_day = now.weekday()
        next_wednesday = now + timedelta(days=abs(week_day - 2))
        translate = request.localizer.translate
        options = {
            'editable': can_edit,
            'droppable': can_edit,
            'eventSources': [{
                'url': absolute_url(context, request, 'get-planning-events.json'),
                'extraParams': {
                    'room': room.__name__
                }
            }],
            'height': '100%',
            'initialDate': next_wednesday.date().isoformat(),
            'firstDay': settings.calendar_first_day,
            'dateClick': 'MyAMS.msc.calendar.addEvent',
            'initialView': 'timeGridWeek',
            'headerToolbar': {
                'center': 'today',
                'right': 'prev,next dayGridMonth,timeGridWeek,timeGridDay,listMonth'
            },
            'allDaySlot': False,
            'slotDuration': f'00:{settings.calendar_slot_duration:#02}:00',
            'slotMinTime': room.start_time.isoformat(),
            'slotMaxTime': room.end_time.isoformat(),
            'buttonText': {
                'today': translate(_("Today")),
                'month': translate(_("Month")),
                'week': translate(_("Week")),
                'day': translate(_("Day")),
                'list': translate(_("List")),
                'all-day': translate(_("All-day")),
                'prev': " « ",
                'next': " » "
            }
        }
        return json.dumps(options)


@view_config(name='get-planning-events.json',
             context=IPlanning, request_type=IPyAMSLayer,
             permission=USE_INTERNAL_API_PERMISSION,
             renderer='json')
def get_planning_events(request):
    """Planning events getter"""
    params = request.params
    room = params.get('room')
    start = params.get('start')
    end = params.get('end')
    if not (room and start and end):
        raise HTTPBadRequest()
    theater = get_parent(request.context, IMovieTheater)
    if theater is None:
        raise HTTPNotFound()
    container = ICinemaRoomContainer(theater, None)
    if (container is None) or (room not in container):
        raise HTTPNotFound()
    catalog = get_utility(ICatalog)
    query = And(Eq(catalog['object_types'], get_interface_base_name(ISession)),
                Eq(catalog['planning_room'], room),
                Le(catalog['planning_start_date'], datetime.fromisoformat(end)),
                Ge(catalog['planning_end_date'], datetime.fromisoformat(start)))
    events = []
    for session in CatalogResultSet(CatalogQuery(catalog).query(query)):
        exporter = request.registry.queryMultiAdapter((session, request), IJSONExporter)
        if exporter is not None:
            events.append(exporter.to_json(with_edit_info=True, edit_context=request.context))
    return events


@viewlet_config(name='planning.transpose',
                layer=IAdminLayer, view=PlanningView,
                manager=ISecondaryActionsViewletManager, weight=10)
class PlanningTransposeAction(JsContextAction):
    """Planning transpose action"""

    icon_class = 'fas fa-recycle'
    hint = _("Transpose calendars")
    hint_placement = 'bottom'

    href = 'MyAMS.msc.calendar.transpose'


@viewlet_config(name='planning.synchronize',
                layer=IAdminLayer, view=PlanningView,
                manager=ISecondaryActionsViewletManager, weight=20)
class PlanningSynchronizeAction(JsContextAction):
    """Planning synchronize action"""

    icon_class = 'fas fa-sync-alt'
    hint = _("Synchronize calendars views")
    hint_placement = 'bottom'

    href = 'MyAMS.msc.calendar.synchronize'


@viewlet_config(name='planning.scroll',
                layer=IAdminLayer, view=PlanningView,
                manager=ISecondaryActionsViewletManager, weight=30)
class PlanningScrollAction(JsContextAction):
    """Planning scrolling action"""

    icon_class = 'fas fa-arrows-alt-v'
    hint = _("Synchronize calendars scrolling")
    hint_placement = 'bottom'

    href = 'MyAMS.msc.calendar.scroll'



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.shared.theater.portlet.interfaces import IAudiencesPortletSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IAudiencesPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/audiences-default.pt', layer=IPyAMSLayer)
class AudiencesPortletBannerRenderer(PortletRenderer):
    """Audiences portlet banner renderer"""

    label = _("MSC: Simple audiences banner (default)")
    weight = 1



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from collections import OrderedDict
from enum import Enum

from zope.container.constraints import containers
from zope.interface import Attribute, Interface
from zope.schema import Bool, Choice, Float, Int, Object, Text, TextLine, URI
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_app_msc.component.address.interfaces import IAddress
from pyams_app_msc.component.admininfo.interfaces import IAdminInfo
from pyams_app_msc.component.banking.interfaces import IBankingAccount
from pyams_app_msc.interfaces import MSC_CONTRIBUTOR_ROLE, MSC_MANAGER_ROLE, MSC_OPERATOR_ROLE, MSC_READER_ROLE
from pyams_content.component.paragraph import IParagraphContainerTarget
from pyams_content.shared.common.interfaces import DEFAULT_CONTENT_WORKFLOW, IBaseSharedTool, IDeletableElement
from pyams_file.schema import FileField, ImageField
from pyams_i18n.schema import I18nTextField
from pyams_portal.interfaces import DESIGNER_ROLE, IPortalPage
from pyams_security.schema import PrincipalsSetField
from pyams_sequence.interfaces import ISequentialIdTarget
from pyams_site.interfaces import ISiteRoot
from pyams_utils.schema import ColorField, MailAddressField
from pyams_workflow.interfaces import IWorkflowPublicationSupport, WORKFLOWS_VOCABULARY

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


#
# Movie theater interfaces
#

MSC_THEATERS_VOCABULARY = 'msc.theaters'


class IMovieTheater(IBaseSharedTool, IDeletableElement, ISequentialIdTarget,
                    IParagraphContainerTarget, IWorkflowPublicationSupport):
    """Movie theater interface"""

    containers(ISiteRoot)

    header = I18nTextField(title=_("Header"),
                           description=_("Site's header is generally displayed in page header"),
                           required=False)

    description = I18nTextField(title=_("Meta-description"),
                                description=_("The site's description is 'hidden' into HTML's "
                                              "page headers; but it can be seen, for example, "
                                              "in some search engines results as content's "
                                              "description; if description is empty, content's "
                                              "header will be used."),
                                required=False)

    shared_content_workflow = Choice(title=_("Workflow name"),
                                     description=_("Name of workflow utility used to manage tool "
                                                   "contents"),
                                     vocabulary=WORKFLOWS_VOCABULARY,
                                     default=DEFAULT_CONTENT_WORKFLOW)

    shared_content_type = Attribute("Shared data content type name")

    logo = ImageField(title=_("Logo"),
                      description=_("This image will be used in documents and email messages"),
                      required=False)

    address = Object(title=_("Address"),
                     description=_("Full theater address"),
                     schema=IAddress,
                     required=False)

    web_address = URI(title=_("Website address"),
                      description=_("URL of theater web site"),
                      required=False)

    contact_email = MailAddressField(title=_("Contact email"),
                                     required=False)

    phone_number = TextLine(title=_("Phone number"),
                            required=False)

    banking_account = Object(title=_("Banking account"),
                             schema=IBankingAccount,
                             required=False)

    admin_info = Object(title=_("Administrative information"),
                        schema=IAdminInfo,
                        required=False)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


MOVIE_THEATER_ROLES = 'msc.theater.roles'


class IMovieTheaterRoles(Interface):
    """Movie theater roles interface"""

    msc_managers = PrincipalsSetField(title=_("Managers"),
                                      description=_("Managers can manage all theater configuration and properties"),
                                      role_id=MSC_MANAGER_ROLE,
                                      required=False)

    msc_operators = PrincipalsSetField(title=_("Operators"),
                                       description=_("Operators can manage theater activities catalog, and "
                                                     "reservations"),
                                       role_id=MSC_OPERATOR_ROLE,
                                       required=False)

    msc_contributors = PrincipalsSetField(title=_("Contributors"),
                                          description=_("Contributors can add activities to theater catalog, but "
                                                        "can't publish them"),
                                          role_id=MSC_CONTRIBUTOR_ROLE,
                                          required=False)

    msc_designers = PrincipalsSetField(title=_("Designers"),
                                       description=_("Designers are users which are allowed to "
                                                     "manage presentation templates"),
                                       role_id=DESIGNER_ROLE,
                                       required=False)

    msc_readers = PrincipalsSetField(title=_("Readers"),
                                     description=_("Readers are guest users which can only view theater properties "
                                                   "and activities catalog"),
                                     role_id=MSC_READER_ROLE,
                                     required=False)


MOVIE_THEATER_SETTINGS_KEY = 'msc.theater.settings'


class CALENDAR_DAYS(Enum):
    """Calendar days enumeration"""
    SUNDAY = 0
    MONDAY = 1
    TUESDAY = 2
    WEDNESDAY = 3
    THURSDAY = 4
    FRIDAY = 5
    SATURDAY = 6


CALENDAR_FIRST_DAY = OrderedDict((
    (CALENDAR_DAYS.SUNDAY, _("Sunday")),
    (CALENDAR_DAYS.MONDAY, _("Monday")),
    (CALENDAR_DAYS.TUESDAY, _("Tuesday")),
    (CALENDAR_DAYS.WEDNESDAY, _("Wednesday")),
    (CALENDAR_DAYS.THURSDAY, _("Thursday")),
    (CALENDAR_DAYS.FRIDAY, _("Friday")),
    (CALENDAR_DAYS.SATURDAY, _("Saturday"))
))


CALENDAR_FIRST_DAY_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in CALENDAR_FIRST_DAY.items()
])


class SESSION_REQUEST_MODE(Enum):
    """Session request modes"""
    FORM = 'form'
    MAILTO = 'mailto'


SESSION_REQUEST_MODE_LABEL = OrderedDict((
    (SESSION_REQUEST_MODE.FORM, _('Use site form')),
    (SESSION_REQUEST_MODE.MAILTO, _('Use mailto link'))
))


SESSION_REQUEST_MODE_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in SESSION_REQUEST_MODE_LABEL.items()
])


class BOOKING_CANCEL_MODE(Enum):
    """Booking cancel modes"""
    FORBIDDEN = 'forbidden'
    MAX_DELAY = 'max-delay'
    NOTICE_PERIOD = 'notice-period'


BOOKING_CANCEL_MODE_LABEL = OrderedDict((
    (BOOKING_CANCEL_MODE.FORBIDDEN, _("Forbidden cancellation")),
    (BOOKING_CANCEL_MODE.MAX_DELAY, _("Max delay after booking")),
    (BOOKING_CANCEL_MODE.NOTICE_PERIOD, _("Minimum notice period before session"))
))


BOOKING_CANCEL_MODE_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v.value, title=t)
    for v, t in BOOKING_CANCEL_MODE_LABEL.items()
])


class IMovieTheaterSettings(Interface):
    """Movie theater settings"""

    calendar_first_day = Choice(title=_("Calendar first week day"),
                                description=_("First week day used in calendars"),
                                vocabulary=CALENDAR_FIRST_DAY_VOCABULARY,
                                required=False,
                                default=CALENDAR_DAYS.SUNDAY.value)

    calendar_slot_duration = Int(title=_("Calendar slot duration"),
                                 description=_("Calendar time slots frequency in week or day views"),
                                 required=False,
                                 min=5,
                                 max=60,
                                 default=30)

    default_session_duration = Int(title=_("Default session duration"),
                                   description=_("This is a default duration, in minutes, of sessions for "
                                                 "which activity has no duration"),
                                   required=False,
                                   default=120)

    session_duration_delta = Int(title=_("Session duration delta"),
                                 description=_("This is a number of minutes which will be added automatically "
                                               "to activities duration when adding new sessions"),
                                 required=False,
                                 default=0)

    allow_session_request = Bool(title=_('Allow session request'),
                                 description=_('When enabled, external partners will be allowed to '
                                               'send requests for new sessions'),
                                 required=True,
                                 default=True)

    session_request_mode = Choice(title=_("Session request mode"),
                                  description=_(""),
                                  required=True,
                                  vocabulary=SESSION_REQUEST_MODE_VOCABULARY,
                                  default=SESSION_REQUEST_MODE.FORM.value)

    reminder_delay = Int(title=_("Reminder delay"),
                         description=_("This is a delay, given in days before a session beginning, at which "
                                       "a reminder message can be sent to booking recipients; leave value "
                                       "empty or set it to 0 to disable reminders messages"),
                         required=False)

    booking_cancel_mode = Choice(title=_("User booking cancel mode"),
                                 description=_("This mode determines how and when a principal can cancel "
                                               "a booking by it's own way"),
                                 vocabulary=BOOKING_CANCEL_MODE_VOCABULARY,
                                 required=True,
                                 default=BOOKING_CANCEL_MODE.FORBIDDEN.value)

    booking_cancel_max_delay = Int(title=_("Cancel max delay"),
                                   description=_("This is the maximum period length after user booking, "
                                                 "given in hours, during which a principal can cancel a "
                                                 "booking by it's own way"),
                                   required=False,
                                   min=0)

    booking_cancel_notice_period = Int(title=_("Cancel notice period"),
                                       description=_("This is the minimum amount of time before session, "
                                                     "given in hours, during which a principal can cancel "
                                                     "a booking by it's own way"),
                                       required=False,
                                       min=0)

    booking_retention_duration = Int(title=_("Booking retention duration"),
                                     description=_("This is the minimum amount of time, given in hours, during which a "
                                                   "a booking is kept 'active' before being archived"),
                                     required=False,
                                     min=0,
                                     default=24)

    quotation_number_format = TextLine(title=_("Quotation number format"),
                                       description=_("This number will be used as formatting string to "
                                                     "create quotations numbers; you can use date elements "
                                                     "like {yyyy} for current year in long format, {yy} for current "
                                                     "year in short format, {mm} for current month, {dd} for current "
                                                     "date, {yinc} for year increment, {minc} for month increment and "
                                                     "{operator} for operator initials"),
                                       default='PREFIX-{yyyy}{yinc}',
                                       required=False)

    quotation_logo = FileField(title=_("Quotation logo"),
                               description=_("This image will be displayed in quotations headers"),
                               required=False)

    def get_logo_color(self):
        """Extract median color from logo"""

    quotation_color = ColorField(title=_("Base color"),
                                 description=_("This color will be used as base color for graphic elements "
                                               "displayed in quotations"),
                                 required=False)

    currency = TextLine(title=_("Currency"),
                        description=_("This is the currency used for all transactions"),
                        required=False)

    vat_rate = Float(title=_("VAT rate"),
                     description=_("This is the VAT rate which will be used in quotations"),
                     required=False)

    def get_quotation_number(self):
        """Get new quotation number"""

    def get_quotation_color(self):
        """Get base quotation color"""


class IMovieTheaterCalendarPortalPage(IPortalPage):
    """Movie theater calendar portal page marker interface"""


class IMovieTheaterMoviesPortalPage(IPortalPage):
    """Movie theater movies list portal page marker interface"""


class IMovieTheaterCatalogPortalPage(IPortalPage):
    """Movie theater catalog portal page marker interface"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_app_msc.feature.profile.skin module

"""

from pyramid.authorization import Authenticated
from pyramid.events import subscriber
from zope.interface import Interface, Invalid, alsoProvides, implementer
from zope.schema import Password

from pyams_app_msc.feature.navigation import NavigationMenuItem
from pyams_app_msc.feature.navigation.interfaces import INavigationViewletManager
from pyams_app_msc.feature.profile.interfaces import IUserProfile
from pyams_app_msc.feature.profile.skin.interfaces import IUserProfileView
from pyams_app_msc.skin import IPyAMSMSCLayer
from pyams_content.root import ISiteRootInfos
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.form import AddForm, EditForm
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_mail.interfaces import IPrincipalMailInfo
from pyams_portal.skin.page import PortalContextIndexPage
from pyams_security.interfaces import ISecurityManager
from pyams_security.interfaces.base import MANAGE_PERMISSION
from pyams_security.interfaces.names import ADMIN_USER_ID, INTERNAL_USER_ID
from pyams_security.interfaces.plugin import ILocalUser, check_password
from pyams_skin.schema.button import ResetButton, SubmitButton
from pyams_template.template import layout_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.registry import get_utility
from pyams_viewlet.viewlet import viewlet_config

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


class UserMenu(NavigationMenuItem):
    """Base user menu"""

    href = None

    @property
    def css_class(self):
        return 'selected' if self.request.view_name == self.href else ''


@layout_config(template='templates/profile-layout.pt',
               layer=IPyAMSMSCLayer)
class ProfileContextIndexPage(PortalContextIndexPage):
    """Profile context index page"""


@viewlet_config(name='user-profile.menu',
                layer=IPyAMSMSCLayer, view=IUserProfileView,
                manager=INavigationViewletManager, weight=10)
class UserProfileMenu(UserMenu):
    """User profile menu"""

    label = _("My profile")
    href = 'my-profile.html'


@ajax_form_config(name='my-profile.html',
                  layer=IPyAMSLayer)
@implementer(IUserProfileView)
class UserProfileEditForm(EditForm, ProfileContextIndexPage):
    """User profile edit form"""

    title = _("My user profile")
    legend = _("Account settings")

    fields = Fields(IUserProfile).omit('active')
    _edit_permission = MANAGE_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        principal_id = self.request.principal.id
        principal_widget = self.widgets.get('principal_id')
        if principal_widget is not None:
            principal_widget.mode = HIDDEN_MODE
            principal_widget.value = principal_id
        phone_number = self.widgets.get('phone_number')
        if phone_number is not None:
            phone_number.object_data = {
                'input-mask': '[+9{3}] [9]9 99 99 99 99'
            }
            alsoProvides(phone_number, IObjectData)
        email = self.widgets.get('email')
        if email is not None:
            email.readonly = 'readonly'
            if not email.value:
                if principal_id in (ADMIN_USER_ID, INTERNAL_USER_ID):
                    infos = ISiteRootInfos(self.request.root)
                    email.value = infos.support_email
                else:
                    sm = get_utility(ISecurityManager)
                    principal_info = sm.get_principal(principal_id, info=False)
                    if principal_info is not None:
                        mail_info = IPrincipalMailInfo(principal_info, None)
                        if mail_info is not None:
                            _title, principal_email = next(mail_info.get_addresses())
                            email.value = principal_email

    def update_actions(self):
        super().update_actions()
        action = self.actions.get('apply')
        if action is not None:
            action.add_class('btn-primary')


@adapter_config(required=(Interface, IPyAMSLayer, UserProfileEditForm),
                provides=IFormContent)
def user_profile_edit_form_content(context, request, form):
    """User profile edit form content getter"""
    return IUserProfile(request)


#
# User password change form
#

@viewlet_config(name='user-password.menu',
                layer=IPyAMSMSCLayer, view=IUserProfileView,
                manager=INavigationViewletManager, weight=20)
class UserPasswordMenu(UserMenu):
    """User password menu"""

    label = _("My password")
    href = 'my-password.html'

    def __new__(cls, context, request, view, manager):
        identity = request.identity
        if (identity is None) or (Authenticated not in identity.get('principals', ())):
            return None
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(request.principal.id)
        if not ILocalUser.providedBy(principal):
            return None
        return NavigationMenuItem.__new__(cls)


class IUserPasswordEditFormFields(IUserProfile):
    """User password change form fields interface"""

    old_password = Password(title=_("Old password"),
                            required=True)

    new_password = Password(title=_("New password"),
                            min_length=8,
                            required=True)

    confirmed_password = Password(title=_("Confirmed new password"),
                                  min_length=8,
                                  required=True)


class IUserPasswordEditFormButtons(Interface):
    """User password edit form buttons interface"""

    change = SubmitButton(name='change',
                          title=_("Change password"))

    reset = ResetButton(name='reset',
                        title=_("Reset"))


@ajax_form_config(name='my-password.html',
                  layer=IPyAMSLayer)
@implementer(IUserProfileView)
class UserPasswordEditForm(AddForm, ProfileContextIndexPage):
    """User password change form"""

    title = _("My password")
    legend = _("Change password")

    fields = Fields(IUserPasswordEditFormFields).select('principal_id', 'old_password',
                                                        'new_password', 'confirmed_password')
    buttons = Buttons(IUserPasswordEditFormButtons)

    _edit_permission = MANAGE_PERMISSION

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        principal_id = self.request.principal.id
        principal_widget = self.widgets.get('principal_id')
        if principal_widget is not None:
            principal_widget.mode = HIDDEN_MODE
            principal_widget.value = principal_id

    def update_actions(self):
        super().update_actions()
        change = self.actions.get('change')
        if change is not None:
            change.add_class('btn-primary')

    @handler(buttons['change'])
    def handle_change(self, action):
        super().handle_add(self, action)

    def create_and_add(self, data):
        data = data.get(self, data)
        sm = get_utility(ISecurityManager)
        principal = sm.get_raw_principal(data.get('principal_id'))
        if ILocalUser.providedBy(principal):
            principal.password = data['new_password']
            return principal


@adapter_config(required=(Interface, IPyAMSLayer, UserPasswordEditForm),
                provides=IFormContent)
def user_password_edit_form_content(context, request, form):
    """User password edit form content getter"""
    return IUserProfile(request)


@subscriber(IDataExtractedEvent, form_selector=UserPasswordEditForm)
def extract_user_password_form_data(event):
    """Check new local user password data"""
    data = event.data
    principal_id = data.get('principal_id')
    sm = get_utility(ISecurityManager)
    principal = sm.get_raw_principal(principal_id)
    if not ILocalUser.providedBy(principal):
        event.form.widgets.errors += (Invalid(_("Can't find local principal!")),)
        return
    if not principal.check_password(data.get('old_password')):
        event.form.widgets.errors += (Invalid(_("Wrong current password!")),)
    new_password = data.get('new_password')
    if new_password:
        try:
            check_password(new_password)
        except Invalid as ex:
            event.form.widgets.errors += (ex,)
        if new_password != data.get('confirmed_password'):
            event.form.widgets.errors += (Invalid(_("User password was not confirmed correctly.")),)
        else:
            del data['confirmed_password']


@adapter_config(required=(Interface, IPyAMSLayer, UserPasswordEditForm),
                provides=IAJAXFormRenderer)
class UserPasswordEditFormRenderer(ContextRequestViewAdapter):
    """User password edit form renderer"""

    def render(self, changes):
        if changes is None:
            return {
                'status': 'reload'
            }
        return {
            'status': 'success',
            'message': self.request.localizer.translate(_("Your password was changed successfully."))
        }



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_app_msc.interfaces import VIEW_CATALOG_PERMISSION
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_content.interfaces import MANAGE_SITE_PERMISSION
from pyams_content.shared.site.zmi.viewlet import SharedSitesMenu
from pyams_content.zmi.viewlet.toplinks import TopTabsViewletManager
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewletmanager_config(name='movie-theaters.menu',
                       context=Interface, layer=IAdminLayer,
                       manager=TopTabsViewletManager, weight=5)
class MovieTheatersMenu(SharedSitesMenu):
    """Movie theaters menu"""

    label = _("My theaters")
    interface = IMovieTheater

    menus_permissions = {
        VIEW_CATALOG_PERMISSION
    }



#
# Copyright (c) 2015-2019 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS MSC application.interfaces module

"""


#
# Custom permissions
#

MANAGE_THEATER_PERMISSION = 'msc.ManageTheater'
'''Permission required to manage theater properties and configuration'''

VIEW_THEATER_PERMISSION = 'msc.ViewTheater'
'''Permission required to view theater properties'''

MANAGE_CATALOG_PERMISSION = 'msc.ManageCatalog'
'''Permission required to manage theater activities catalog and reservations'''

VIEW_CATALOG_PERMISSION = 'msc.ViewCatalog'
'''Permission required to read theater activities catalog and reservations'''

CREATE_BOOKING_PERMISSION = 'msc.CreateBooking'
'''Permission required to create a booking request for a theater activity'''

VIEW_PLANNING_PERMISSION = 'msc.ViewPlanning'
'''Permission required to view sessions planning'''

MANAGE_PLANNING_PERMISSION = 'msc.ManagePlanning'
'''Permission required to manage sessions planning'''

VIEW_BOOKING_PERMISSION = 'msc.ViewBooking'
'''Permission required to view booking information for a theater activity'''

MANAGE_BOOKING_PERMISSION = 'msc.ManageBooking'
'''Permission required to manage bookings on a theater activity'''

CANCEL_BOOKING_PERMISSION = 'msc.CancelBooking'
'''Permission required to cancel a booking on a theater activity'''


#
# Custom roles
#

MSC_SITES_MANAGER_ROLE = 'msc.SitesManager'
'''Site manager role can create and manage sites and theaters'''

MSC_MANAGER_ROLE = 'msc.Manager'
'''Manager role has all permissions on a theater'''

MSC_OPERATOR_ROLE = 'msc.Operator'
'''Operator role can manage theater activities catalog and reservations'''

MSC_CONTRIBUTOR_ROLE = 'msc.Contributor'
'''Contributor role can prepare activities for catalog, but can't publish them'''

MSC_READER_ROLE = 'msc.Reader'
'''Reader role have read-only access to theater catalog and reservations'''

MSC_CLIENT_ROLE = 'msc.Client'
'''Client role can ask for reservations on planned theater activities'''

MSC_OWNER_ROLE = 'msc.Owner'
'''Owner role can manage it's reservations on planned theater activities'''



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from datetime import date, datetime, timedelta, timezone

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Eq, Ge, Lt
from zope.intid.interfaces import IIntIds

from pyams_app_msc.feature.booking import IBookingContainer
from pyams_app_msc.feature.booking.interfaces import BOOKING_STATUS
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.shared.theater import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_template.template import template_config
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import INavigationViewletManager
from pyams_zmi.view import InnerAdminView
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


@viewlet_config(name='today-program.menu',
                context=IMovieTheater, layer=IAdminLayer,
                manager=INavigationViewletManager, weight=10,
                permission=VIEW_SYSTEM_PERMISSION)
class TodayProgramMenu(NavigationMenuItem):
    """Today program menu"""

    label = _("Today program")
    icon_class = 'fas fa-tasks'
    href = '#today-program.html'


@pagelet_config(name='today-program.html',
                context=IMovieTheater, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/today-program.pt',
                 layer=IAdminLayer)
class TodayProgramView(InnerAdminView):
    """Today program view"""

    title = _("Sessions planned for today")

    def get_sessions(self):
        """Today sessions getter"""
        now = tztime(datetime.now(timezone.utc))
        today_start = tztime(datetime.combine(date.today(), datetime.min.time()))
        today_end = today_start + timedelta(days=1)
        catalog = get_utility(ICatalog)
        intids = get_utility(IIntIds)
        params = And(Eq(catalog['object_types'], get_interface_base_name(ISession)),
                     Eq(catalog['parents'], intids.register(self.context)),
                     Lt(catalog['planning_start_date'], today_end),
                     Ge(catalog['planning_end_date'], now))
        yield from CatalogResultSet(CatalogQuery(catalog).query(params, sort_index='planning_start_date'))

    @staticmethod
    def get_bookings(session):
        """Session bookings getter"""
        yield from filter(lambda x: x.status == BOOKING_STATUS.ACCEPTED.value,
                          IBookingContainer(session).values())

    def get_groups(self, booking):
        """Booking groups getter"""
        translate = self.request.localizer.translate
        if booking.nb_groups > 1:
            return translate(_("{} groups")).format(booking.nb_groups)
        return translate(_("1 group"))
